#ifndef CCAR_H
#define CCAR_H

#include <SDL/SDL.h>
#include <SDL/SDL_rotozoom.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_image.h>
#include <vector>
#include <math.h>

#include "CSinCosTable.h"
#include "Functions.h"
#include "fixed_class.h"
#include "Race.h"
#include "CCamera.h"







using namespace fixedpoint;
using namespace std;


struct SPoint
{
    long X;
    long Y;
    long VelX;
    long VelY;
    int Angle;
    bool Bouncing;
    int JumpHeight;
};

class CCar {
    protected:
        int PId;
        vector<SPoint> Points;
        bool PSlippingWater,PBounce,PJumping,PLanding,PResetting,PLanded,PReplayRecording,PHasBounced,PCanMove;
        SDL_Surface *PSurface,*POriginalSurface,*PMaskSurface, *POriginalMaskSurface,*PShadow;
        int PAccelStep;
        int PAngle,PAngleInc,PMaxTurnAngle,PTurnAngle;
        int PInitialX,PInitialY,PX,PY,PVelX,PVelY,PBounces,PResetSteps,PBounceSteps;
        int PSpeed,PAcceleration,PMaxSpeed,PMaxAcceleration,PMaxJumpHeight,PJumpHeight,PJumpIncrease,PMaxSpeedNitro,PMaxAccelerationNitro;
        bool PVisible;
        bool POverBridge;
        bool PInOnBridge;
        virtual void UpdateAngle();
        bool PCheckPoint1Passed,PCheckPoint2Passed,PFinishedPassed ;
        bool POverLogicalPortVertEnter,POverLogicalPortVertLeave,POverLogicalPortHorzEnter,POverLogicalPortHorzLeave,PLogicalPortHorizontal,PLogicalPortHorzValue,PLogicalPortVertValue;
        int PLastPort,PLastPortIndex,PLogicalPortVertEnteredAngleEnter,PLogicalPortHorzEnteredAngleEnter,PLogicalPortVertEnteredAngleLeave,PLogicalPortHorzEnteredAngleLeave,PLaps;
    public:
        CCar();
        CCar(char* filename,char* filenamemask,char* filenameshadow,int Xin,int Yin,int Angle);
        CCar(char* filename,char* filenamemask,char* filenameshadow,int Xin,int Yin,int Angle,double MaxSpeed, double MaxAcceleration,double AccelStep);
        CCar(char* filename,char* filenamemask,char* filenameshadow,int Xin,int Yin,int Angle,double MaxSpeed, double MaxAcceleration,double AccelStep,double MaxTurnAngle,double AngleInc );
        virtual ~CCar();
        virtual void Move();
        void SetCollisionAngle(CCar *Car);
        inline SDL_Surface* GetMaskSurface(){return PMaskSurface;}
        inline float GetMaxAcceleration(){return fix2float<Precission>(PMaxAcceleration);}
        inline void SetMaxAcceleration(float MaxAcceleration){ PMaxAcceleration= float2fix<Precission>(MaxAcceleration);}
        inline float GetAcceleration(){return fix2float<Precission>(PAcceleration);}

        inline int GetLaps(){return PLaps;}
        inline void SetLaps(int Laps){PLaps = Laps;}

        inline int GetLogicalPortVertEnteredAngleEnter() {return PLogicalPortVertEnteredAngleEnter;}
        inline int GetLogicalPortHorzEnteredAngleEnter() {return PLogicalPortHorzEnteredAngleEnter;}
        inline void SetLogicalPortHorzEnteredAngleEnter(int LogicalPortHorzEnteredAngleEnter) {PLogicalPortHorzEnteredAngleEnter= LogicalPortHorzEnteredAngleEnter;}
        inline void SetLogicalPortVertEnteredAngleEnter(int LogicalPortVertEnteredAngleEnter) {PLogicalPortVertEnteredAngleEnter= LogicalPortVertEnteredAngleEnter;}

        inline int GetLogicalPortVertEnteredAngleLeave() {return PLogicalPortVertEnteredAngleLeave;}
        inline int GetLogicalPortHorzEnteredAngleLeave() {return PLogicalPortHorzEnteredAngleLeave;}
        inline void SetLogicalPortHorzEnteredAngleLeave(int LogicalPortHorzEnteredAngleLeave) {PLogicalPortHorzEnteredAngleLeave= LogicalPortHorzEnteredAngleLeave;}
        inline void SetLogicalPortVertEnteredAngleLeave(int LogicalPortVertEnteredAngleLeave) {PLogicalPortVertEnteredAngleLeave= LogicalPortVertEnteredAngleLeave;}



        inline bool GetInOnBridge () { return PInOnBridge;}
        inline void SetInOnBridge(bool InOnBridge) {PInOnBridge = InOnBridge;}

        inline bool GetLogicalPortHorizontal () { return PLogicalPortHorizontal;}
        inline void SetLogicalPortHorizontal(bool LogicalPortHorizontal) {PLogicalPortHorizontal = LogicalPortHorizontal;}

        inline void SetLogicalPortHorzValue(bool LogicalPortHorzValue) {PLogicalPortHorzValue = LogicalPortHorzValue;}
        inline void SetLogicalPortVertValue(bool LogicalPortVertValue) {PLogicalPortVertValue = LogicalPortVertValue;}

        inline bool GetLogicalPortVertValue() {return PLogicalPortVertValue;}
        inline bool GetLogicalPortHorzValue() {return PLogicalPortHorzValue;}

        inline int GetLastPortIndex(){return PLastPortIndex;}
        inline void SetLastPortIndex(int LastPortIndex){PLastPortIndex = LastPortIndex;}
        inline int GetLastPort(){return PLastPort;}
        inline void SetLastPort(int LastPort){PLastPort = LastPort;}
        inline bool GetOverLogicalPortHorzEnter () {return POverLogicalPortHorzEnter;}
        inline bool GetOverLogicalPortVertEnter () { return POverLogicalPortVertEnter;}
        inline void SetOverLogicalPortHorzEnter (bool OverLogicalPortHorzEnter) {POverLogicalPortHorzEnter = OverLogicalPortHorzEnter;}
        inline void SetOverLogicalPortVertEnter (bool OverLogicalPortVertEnter) {POverLogicalPortVertEnter = OverLogicalPortVertEnter;}

        inline bool GetOverLogicalPortHorzLeave () {return POverLogicalPortHorzLeave;}
        inline bool GetOverLogicalPortVertLeave () { return POverLogicalPortVertLeave;}
        inline void SetOverLogicalPortHorzLeave (bool OverLogicalPortHorzLeave) {POverLogicalPortHorzLeave = OverLogicalPortHorzLeave;}
        inline void SetOverLogicalPortVertLeave (bool OverLogicalPortVertLeave) {POverLogicalPortVertLeave = OverLogicalPortVertLeave;}

        inline void SetCheckPoint1Passed(bool CheckPoint1Passed) { PCheckPoint1Passed = CheckPoint1Passed;}
        inline void SetCheckPoint2Passed(bool CheckPoint2Passed) { PCheckPoint2Passed = CheckPoint2Passed;}
        inline void SetFinishPassed(bool FinishedPassed) { PFinishedPassed = FinishedPassed;}
        inline bool GetCheckPoint1Passed() {return PCheckPoint1Passed;}
        inline bool GetCheckPoint2Passed() {return PCheckPoint2Passed;}
        inline bool GetFinishPassed() {return PFinishedPassed;}
        inline void SetOverBridge(bool OverBridge) {POverBridge = OverBridge;}
        inline bool GetOverBridge() { return POverBridge;}
        void TurnRight();
        void TurnLeft();
        void SetAngle(int Angle);
        void Bounce();
        void Jump();
        void SlipWater(int AngleInc);
        void SlipWaterStop();
        inline int GetAngle() { return PAngle;}
        inline bool IsJumping() {return PJumping;}
        inline bool IsLanding() {return PLanding;}
        inline bool IsResetting() {return PResetting;}
        inline bool IsBouncing() {return PBounce;}
        SDL_Rect GetRect();
        virtual void Draw(SDL_Surface *Surface, CCamera *Camera);
        inline double GetX() {return fix2float<Precission>(PX);}
        inline double GetY() {return fix2float<Precission>(PY) ;}
        inline int GetIntX() {return (int)floor(fix2float<Precission>(PX));}
        inline int GetIntY() {return (int)floor(fix2float<Precission>(PY));}
        inline double GetNextX() {return fix2float<Precission>(PX+PVelX);}
        inline double GetNextY() {return fix2float<Precission>(PY+PVelY);}
        void Accelerate();
        void AccelerateNitro();
        void DeAccelerate();
        inline bool IsLanded () {return PLanded;};
        bool Intersects(SDL_Rect *Rect);
        void ResetCar();
        inline void ResetLanded() { PLanded = false;}
        inline void ResetCarStop(){ PResetting = false;}
        bool CheckPixelCollision(SDL_Rect Rect);
        bool CheckPixelCollisionPoint(int X, int Y);
        inline float GetSpeed(){return fix2float<Precission>(PSpeed);}
        inline void SetSpeed(float Speed){PSpeed = float2fix<Precission>(Speed);}
        inline float GetMaxSpeed(){return fix2float<Precission>(PMaxSpeed);}
        inline void SetMaxSpeed(float MaxSpeed){ PMaxSpeed = float2fix<Precission>(MaxSpeed);}
        inline void ReverseVelocity(){ PVelX = -PVelX; PVelY = - PVelY;}
        inline void ReverseSpeed(){PSpeed = -PSpeed;}
        inline vector<SPoint> GetReplayData() {return Points;}
        void SetReplayData(vector<SPoint> ReplayData);
        void EraseReplayData();
        virtual void ResetReplay() {};
        bool CheckPositionCollision(SDL_Rect Rect);
        inline void Hide(){PVisible = false;}
        inline void UnHide(){PVisible = true;}
        inline bool HasBounced(){return PHasBounced;}
        inline void ResetHasBounced() {PHasBounced=false;}
        float GetBounceToY();
        float GetBounceToX();
        void SetAngleNoUpdate(int Angle);
        void StopBounce();
        bool CheckMaskCollision(SDL_Rect Position);
        bool CollideRect(SDL_Rect Rect);
        void ResetTurnAngle();
        inline void SetCanMove(bool CanMove) {PCanMove = CanMove;}
        inline bool GetCanMove() {return PCanMove;}
        inline void ResetToInitialPosition() { PX = PInitialX; PY=PInitialY; }
        inline void SetX(int X) { PX = X << Precission;}
        inline void SetY(int Y) { PY = Y << Precission;}
};


#endif
