
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_framerate.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <time.h>
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fstream>

#include "CCamera.h"
#include "gp2x.h"
#include "caanoo.h"
#include "CInput.h"
#include "Race.h"
#include "CCar.h"
#include "Functions.h"
#include "CSlideDoorLeft.h"
#include "CSlideDoorRight.h"
#include "CSlideDoorUp.h"
#include "CSlideDoorDown.h"
#include "CPartsManager.h"
#include "CReplayCar.h"
#include "CFinishUp.h"
#include "CFinishLeft.h"
#include "CFinishRight.h"
#include "CFinishDown.h"
#include "CCheckPoint1Vert.h"
#include "CCheckPoint2Vert.h"
#include "CCheckPoint1Horz.h"
#include "CCheckPoint2Horz.h"
#include "CLogicalPortHorzEnter.h"
#include "CLogicalPortHorzLeave.h"
#include "CLogicalPortVertEnter.h"
#include "CLogicalPortVertLeave.h"
#include "limits.h"

enum GameStates {GSQuit,GSTest,GSLevelSelect};
struct SObstacle {
    SDL_Rect Location;
    int Type;
};

CPartsManager *GPartsManager = new CPartsManager();
CCar *GCar =NULL;
CReplayCar *GReplayCar=NULL;
CCamera *Camera=NULL;
int GLevel = 1,GPreviousEngineSound=0;
int Volume=128,StartScreenX=0,StartScreenY=0,InitialStartScreenX=20,InitialStartScreenY=24;
bool TvOutMode =false;
Mix_Chunk *EngineSounds[10];


SDL_Rect ScreenRect;

SDL_Surface *Screen,*Buffer,*LevelBackground=NULL,*LevelMask=NULL,*LevelTop=NULL,*LevelMiddle=NULL,*Kernel=NULL,*BlurredMask=NULL;
TTF_Font *font;
bool GlobalSoundEnabled=true;
FPSmanager Fpsman;
GameStates GameState=GSTest;
Uint32 FPSTicks =0;
int FrameCounter=0, Fps=0;
char FpsText[250];
SDL_Color FpsTextColor = {255,0,255,0};
int Angle=0;
double StartX =WINDOW_WIDTH/2,StartY = WINDOW_HEIGHT/2;
SObstacle Obstacles[1] = {{{260,135,45,20},0}};
Mix_Music *Music;
Mix_Chunk *Sound;
int FinishDirection=DFINISHUPID;
Uint32 LapTime=99999,BestLapTime=999999;
bool CarBouncedBridge=false,PixelBounce=false,MaySwitchPorts=true,OverLogicalPort1=false,OverLogicalPort2=false,LogicalPortHorizontal=false,CanResetReplay=false,OverBridge=false;
int PrevCollisionIndex= -1,PrevCollisionPortId=-1,TmpX=0,TmpY=0,LogicalPort1EnteredAngle=-1,LogicalPort2EnteredAngle=-1,LogicalPort1LeftAngle=-1,LogicalPort2LeftAngle=-1,LastLogicalPortEntered=-1;


int TestPixelMask(int X, int Y, SDL_Surface *Mask)
{
    if (SDL_MUSTLOCK(Mask))
        SDL_LockSurface(Mask);
    Uint32 Color = GetPixel(Mask,X,Y);

    //printf("%d %d %x\n",X,Y,Color);
    if (SDL_MUSTLOCK(Mask))
        SDL_UnlockSurface(Mask);
    //printf("%x\n",Color);
    if (Color == SDL_MapRGB(Mask->format,0,0,0))
        return 0;
    else
        return 1;
}

void SetVolume(const int VolumeIn)
{
	Mix_Volume(0,VolumeIn);
	Mix_Volume(1,VolumeIn);
	Mix_VolumeMusic(VolumeIn);
}

void LoadSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","r");
 	if(SettingsFile)
 	{
		fscanf(SettingsFile,"Volume=%d\n",&Volume);
		SetVolume(Volume);
		if(TvOutMode)
		{
            fscanf(SettingsFile,"StartScreenX=%d\n",&StartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&StartScreenY);
		}
		else
        {
            fscanf(SettingsFile,"StartScreenX=%d\n",&InitialStartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&InitialStartScreenY);
            StartScreenX = 0;
            StartScreenY = 0;
        }
		fclose(SettingsFile);
 	}
 	else
 	{
		SetVolume(128);
		StartScreenX = (Screen->w - Buffer->w) / 2;
        StartScreenY = (Screen->h - Buffer->h) / 2;
	}
}

void SaveSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","w");
 	if(SettingsFile)
 	{
		fprintf(SettingsFile,"Volume=%d\n",Volume);
        if(TvOutMode)
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",StartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",StartScreenY);
        }
        else
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",InitialStartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",InitialStartScreenY);
        }
		fclose(SettingsFile);
#ifdef GP2X
		sync();
#endif
 	}
}


void UnLoadLevel()
{
    int Teller;
    GPartsManager->RemoveAll();

//    for (Teller = 0;Teller<NUMAICARS;Teller++)
//    {
//        if(GAiCars[Teller])
//        {
//            delete GAiCars[Teller];
//            GAiCars[Teller] = NULL;
//        }
//    }
    if(GCar)
    {
        delete GCar;
        GCar = NULL;
    }
    if(GReplayCar)
    {
        delete GReplayCar;
        GReplayCar = NULL;
    }


    if(LevelTop)
    {
        SDL_FreeSurface(LevelTop);
        LevelTop=NULL;
    }
    if(BlurredMask)
    {
        SDL_FreeSurface(BlurredMask);
        BlurredMask=NULL;
    }
    if(LevelBackground)
    {
        SDL_FreeSurface(LevelBackground);
        LevelBackground=NULL;
    }
    if(LevelMask)
    {
        SDL_FreeSurface(LevelMask);
        LevelMask=NULL;
    }
    if(LevelMiddle)
    {
        SDL_FreeSurface(LevelMiddle);
        LevelMiddle=NULL;
    }
}

void GetRectColor(SDL_Surface *Mask, Uint32 Color, int StartX1 ,int  StartY1, SDL_Rect *Rect)
{
   // printf("Checking for Color %d at %d,%d\n",Color,StartX,StartY);
    int X,Y;
    Rect->x = StartX1;
    Rect->y = StartY1;
    X = StartX1;
    Y = StartY1;
    while (GetPixel(Mask,X,Y) == Color)
    {
        X++;
    }
    Rect->w = X - StartX1;
    X = StartX1;
    Y = StartY1;
    while (GetPixel(Mask,X,Y) == Color)
    {
        Y++;
    }
    Rect->h = Y - StartY1;
}

void ParseMask(SDL_Surface *Mask,CPartsManager *PartsManager)
{
    int X,Y,Teller;
    Uint32 Color;
    SDL_Rect Rect;

    if(Camera)
        delete Camera;

    if(SDL_MUSTLOCK(Mask))
        SDL_LockSurface(Mask);

    for(X = 0; X < Mask->w;X++)
        for(Y = 0;Y<Mask->h;Y++)
        {
          //  printf("Checking %d,%d\n",X,Y);
            Color = GetPixel(Mask,X,Y);
           /* if(Color == SDL_MapRGB(Mask->format,0,0,255))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                   // GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CWayPoint(X,Y,1,1,NULL));
                    printf("Added Waypoint at X:%d Y:%d W:%d H:%d\n",X,Y,1,1);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,192,192,192))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                   // GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CWayPoint3(X,Y,1,1,NULL));
                    printf("Added Waypoint 3 at X:%d Y:%d W:%d H:%d\n",X,Y,1,1);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,255,0,255))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                  //  GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CWayPoint2(X,Y,1,1,NULL));
                    printf("Added Waypoint2 at X:%d Y:%d W:%d H:%d\n",X,Y,1,1);
                }
            }
            */

            if(Color == SDL_MapRGB(Mask->format,255,255,10))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CFinishUp(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    GCar = new CCar("./graphics/greencar.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x+(Rect.w/8),Rect.y+Rect.h+1,270,3.0,0.025,0.008,4.0,0.4);
                    GReplayCar = new CReplayCar("./graphics/greencarreplay.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x+(Rect.w/8),Rect.y+Rect.h+1,270);
                    printf("Added Finish Up at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,255,255,20))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CFinishDown(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    GCar = new CCar("./graphics/greencar.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x+Rect.w/8,Rect.y-1,90,3.0,0.025,0.008,4.0,0.4);
                    GReplayCar = new CReplayCar("./graphics/greencarreplay.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x+Rect.w/8,Rect.y-1,90);
                    printf("Added Finish Down at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,255,255,30))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CFinishLeft(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    GCar = new CCar("./graphics/greencar.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x+Rect.w+1,Rect.y+Rect.h/8,180,3.0,0.025,0.008,4.0,0.4);
                    GReplayCar = new CReplayCar("./graphics/greencarreplay.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x+Rect.w+1,Rect.y+Rect.h/8,180);
                    printf("Added Finish Left at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,255,255,40))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CFinishRight(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    GCar = new CCar("./graphics/greencar.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x-1,Rect.y+Rect.h/8,0,3.0,0.025,0.008,4.0,0.4);
                    GReplayCar = new CReplayCar("./graphics/greencarreplay.png","./graphics/greencarmask.png","./graphics/carshadow.png",Rect.x-1,Rect.y+Rect.h/8,0);
                    printf("Added Finish Right at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }

            }

            if(Color == SDL_MapRGB(Mask->format,0,255,10))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {
                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CCheckPoint1Vert(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    printf("Added Checkpoint 1 Vertical at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,0,255,20))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CCheckPoint1Horz(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    printf("Added Checkpoint 1 Horizontal at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,0,255,30))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CCheckPoint2Vert(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    printf("Added Checkpoint 2 Vertical at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,0,255,40))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CCheckPoint2Horz(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    printf("Added Checkpoint 2 horizontal at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,128,128,10))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CSlideDoorUp(Rect.x,Rect.y,Rect.w,Rect.h,0.5,0.5,false,1000,NULL));
                    printf("Added Sliding door up at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,128,128,20))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CSlideDoorDown(Rect.x,Rect.y,Rect.w,Rect.h,0.5,0.5,false,1000,NULL));
                    printf("Added Sliding door down at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,128,128,30))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {

                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CSlideDoorLeft(Rect.x,Rect.y,Rect.w,Rect.h,0.5,0.5,false,1500,NULL));
                    printf("Added Sliding door left at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,128,128,40))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {
                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CSlideDoorRight(Rect.x,Rect.y,Rect.w,Rect.h,0.5,0.5,false,1500,NULL));
                    printf("Added Sliding door right  at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,0,128,10))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {
                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CLogicalPortVertEnter(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                      printf("Added Logical Port 1 Vertical at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,0,128,20))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {
                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CLogicalPortHorzEnter(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    printf("Added Logical Port 1 Horizontal at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,0,128,30))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {
                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CLogicalPortVertLeave(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    printf("Added Logical Port 2 Vertical at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

            if(Color == SDL_MapRGB(Mask->format,0,128,40))
            {
                if(PartsManager->CollidePoint(X,Y) == -1)
                {
                    GetRectColor(Mask,Color,X,Y,&Rect);
                    PartsManager->Add(new CLogicalPortHorzLeave(Rect.x,Rect.y,Rect.w,Rect.h,NULL));
                    printf("Added Logical Port 2 Horizontal  at X:%d Y:%d W:%d H:%d\n",X,Y,Rect.w,Rect.h);
                }
            }

        }

    if(SDL_MUSTLOCK(Mask))
        SDL_UnlockSurface(Mask);

}

void LoadLevel(int Level)
{
    SDL_Surface *Tmp;
    char filename [512];
    UnLoadLevel();



    /*
    sprintf(filename,"./graphics/kernel.png",Level);
    Tmp = IMG_Load(filename);
    Kernel = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(filename,"./graphics/level%dmask.png",Level);
    Tmp = IMG_Load(filename);
    BlurredMask =Blur(Tmp,Kernel);
    SDL_FreeSurface(Tmp);

    SDL_SaveBMP(BlurredMask,"./export/blur.bmp");
    SDL_FreeSurface(Kernel);*/

    sprintf(filename,"./graphics/level%dbackground.png",Level);
    Tmp = IMG_Load(filename);
    LevelBackground =SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(filename,"./graphics/level%dmask.png",Level);
    Tmp = IMG_Load(filename);
    LevelMask =SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);



    sprintf(filename,"./graphics/level%dtoplayer.png",Level);
    Tmp = IMG_Load(filename);
    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Tmp->format,255,0,255));
    LevelTop = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(filename,"./graphics/level%dmiddlelayer.png",Level);
    Tmp = IMG_Load(filename);
    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Tmp->format,255,0,255));
    LevelMiddle = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    ParseMask(LevelMask,GPartsManager);

    Camera = new CCamera(WINDOW_WIDTH,WINDOW_HEIGHT,LevelMask->w,LevelMask->h);
    Camera->SetPos(GCar->GetIntX(), GCar->GetIntY());
}


void HandleInput(CInput *Input)
{
    Input->Update();

    CReplayCar *Repc;
#if defined (GP2X)
    if(Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->JoystickHeld[0][CAANOO_BTN_HOME])
#else
    if(Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[0][CAANOO_BTN_HOME])
        GameState = GSQuit;
#endif


#if defined (GP2X)
    if(Input->JoystickHeld[0][GP2X_BUTTON_L]  || Input->KeyboardHeld[SDLK_l])
    {
        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT]  || Input->KeyboardHeld[SDLK_LEFT]) || Input->JoystickHeld[1][JOYSTICK_LEFT])
        {
            if(StartScreenX - 2 >= 0)
                StartScreenX -=2;
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->KeyboardHeld[SDLK_RIGHT]) || Input->JoystickHeld[1][JOYSTICK_RIGHT])
        {
            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                StartScreenX +=2;
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_UP]  || Input->KeyboardHeld[SDLK_UP]) || Input->JoystickHeld[1][JOYSTICK_UP])
        {
            if(StartScreenY - 2 >=0)
                StartScreenY -=2;
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->KeyboardHeld[SDLK_DOWN]) || Input->JoystickHeld[1][JOYSTICK_DOWN])
        {
            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                StartScreenY +=2;
            Input->Delay();
        }
    }
#endif

#if !defined (GP2X) || !defined(CAANOO)
    if(Input->KeyboardHeld[SDLK_r])
    {
        if(CanResetReplay)
        {
            GReplayCar->SetReplayData(GCar->GetReplayData());
            GReplayCar->ResetReplay();
            GCar->EraseReplayData();
            CanResetReplay = false;
            GReplayCar->UnHide();
        }
    }

    if(Input->KeyboardHeld[SDLK_e])
    {
        if(!CanResetReplay)
        {
            Mix_PlayChannel(-1,Sound,0);
            CanResetReplay = true;
        }


    }

    if(Input->KeyboardHeld[SDLK_t])
    {
        Repc = (CReplayCar*) GReplayCar;
        Repc->Step(1);
    }

    if(Input->KeyboardHeld[SDLK_y])
    {
       Repc = (CReplayCar*) GReplayCar;
        Repc->Step(-1);
    }
#endif

#if defined (GP2X)
    if(Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][GP2X_BUTTON_A] )
#else
    if(Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][CAANOO_BTN_X])
#endif
    {
            GCar->Accelerate();
            //Car->AccelerateNitro();
            //Input->Delay();
    }
    else
       GCar->DeAccelerate();

   // if(!GCar->IsResetting())
    //    GCar->DeAccelerate();

#if defined (GP2X)
    if (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][GP2X_BUTTON_X] )
#else
    if (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][CAANOO_BTN_A])
#endif
    {
        GCar->DeAccelerate();
    }


#if defined (GP2X)
    if(Input->JoystickHeld[0][GP2X_BUTTON_B] || Input->KeyboardHeld[SDLK_SPACE] || Input->JoystickHeld[1][0])
#else
    if(Input->KeyboardHeld[SDLK_SPACE] || Input->JoystickHeld[1][0] || Input->JoystickHeld[0][CAANOO_BTN_B])
#endif
    {
        GCar->Jump();
    }
#if defined (GP2X)
    if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->KeyboardHeld[SDLK_LEFT]))
#else
    if(Input->Ready() && (Input->KeyboardHeld[SDLK_LEFT] || Input->JoystickHeld[0][JOYSTICK_LEFT]))
#endif
    {
        GCar->TurnLeft();
    }
    else
#if defined (GP2X)
    if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->KeyboardHeld[SDLK_RIGHT]))
#else
    if(Input->Ready() && (Input->KeyboardHeld[SDLK_RIGHT] || Input->JoystickHeld[0][JOYSTICK_RIGHT]))
#endif
    {
        GCar->TurnRight();
    }
    else
        GCar->ResetTurnAngle();

#if defined (GP2X)
    if(Input->Ready() && (Input->KeyboardHeld[SDLK_PAGEUP] || Input->JoystickHeld[0][GP2X_BUTTON_R] ))
#else
    if(Input->Ready() && (Input->KeyboardHeld[SDLK_PAGEUP] || Input->JoystickHeld[0][CAANOO_BTN_R]))
#endif
    {

        GameState = GSLevelSelect;
    }
}

void DrawLevel(CCar * Car, CReplayCar * ReplayCar, SDL_Surface *Background, SDL_Surface *MiddleLayer,SDL_Surface *TopLayer,SDL_Surface *Dst,CPartsManager *PartsManager)
{
    char Tekst[512],Teller;
    SDL_BlitSurface(LevelBackground,&(Camera->CameraRect),Dst,NULL);
    PartsManager->Draw(Dst);


    if(ReplayCar)
        ReplayCar->Draw(Dst,Camera,Car);

    Car->Draw(Dst,Camera);
//    for(Teller=0;Teller<NUMAICARS;Teller++)
//        AiCars[Teller]->Draw(Dst);
    if( !Car->GetInOnBridge())
        SDL_BlitSurface(MiddleLayer,NULL,Dst,NULL);
    SDL_BlitSurface(TopLayer,NULL,Dst,NULL);

    SDL_Color Color = {255,255,255,255};

    sprintf(Tekst,"Laps :%d - Time:%2.2f Best:%2.2f" ,Car->GetLaps(),(float)((SDL_GetTicks() - LapTime))/1000,(float)((BestLapTime))/1000);
   // sprintf(Tekst,"Laps :%d CA:%d CAi:%d V:%d %d H:%d %d",Car->GetLaps(),Car->GetAngle(),AiCar->GetAngle(),AiCar->GetLogicalPortVertEnteredAngleEnter(),AiCar->GetLogicalPortVertEnteredAngleLeave(),AiCar->GetLogicalPortHorzEnteredAngleEnter(),AiCar->GetLogicalPortHorzEnteredAngleLeave());
    WriteText(Dst,font,Tekst,strlen(Tekst),0,0,0,Color,true);
    //rectangleRGBA(Dst,Car->GetRect().x,Car->GetRect().y,Car->GetRect().x+Car->GetRect().w,Car->GetRect().y+Car->GetRect().h,255,0,0,255);


}

void HandleMusicLoop()
{
     if(true)
        if(GlobalSoundEnabled)
            if(!Mix_PlayingMusic())
            {
                //SelectedMusic = 0;
                Mix_PlayMusic(Music,0);
                SetVolume(64);
            }
}

void HandleSound(CCar *Car)
{
    if((int)trunc(Car->GetSpeed() / Car->GetMaxSpeed() * 9) != GPreviousEngineSound )
    {
        Mix_HaltChannel(1);
        GPreviousEngineSound = (int)trunc(Car->GetSpeed() / Car->GetMaxSpeed() * 9);
        Mix_PlayChannel(1,EngineSounds[GPreviousEngineSound],-1);
    }




}
void SwitchLogicalPortsLeave(CCar *Car)
{
    printf("SwitchLogicalPortsLeave\n");
    if(!Car->GetOverLogicalPortHorzLeave() && Car->GetLogicalPortHorzEnteredAngleLeave() !=-1 && Car->GetLogicalPortHorzEnteredAngleEnter() ==-1 && !Car->GetOverBridge() && Car->GetLastPort() == 4)
    {
        printf("5\n");
        Car->SetLogicalPortVertValue(false);
        Car->SetOverBridge(true);
        Car->SetInOnBridge(!Car->GetInOnBridge());
    }
    else
        if(Car->GetOverLogicalPortHorzLeave() && Car->GetLogicalPortHorzEnteredAngleLeave() !=-1 && Car->GetLogicalPortHorzEnteredAngleEnter() ==-1 && Car->GetOverBridge() && Car->GetLastPort() == 4)
        {
            if((Car->GetAngle() > 180 && Car->GetLogicalPortHorzEnteredAngleLeave() < 180) || (Car->GetAngle() < 180 && Car->GetLogicalPortHorzEnteredAngleLeave() > 180))
            {
                printf("5-1\n");
                Car->SetLogicalPortVertValue(true);
                Car->SetOverBridge(false);
                Car->SetInOnBridge(!Car->GetInOnBridge());
                Car->SetLogicalPortHorzEnteredAngleLeave(-1);
                //Car->SetOverLogicalPortHorzEnter(false);
            }
        }

    if(!Car->GetOverLogicalPortVertLeave() && Car->GetLogicalPortVertEnteredAngleLeave() !=-1 && Car->GetLogicalPortVertEnteredAngleEnter() ==-1 && !Car->GetOverBridge() && Car->GetLastPort() == 3)
    {
        printf("6\n");
        Car->SetLogicalPortHorzValue(false);
        Car->SetOverBridge(true);
       // Car->SetInOnBridge(!Car->GetInOnBridge());

    }
    else
        if(Car->GetOverLogicalPortVertLeave() && Car->GetLogicalPortVertEnteredAngleLeave() !=-1 && Car->GetLogicalPortVertEnteredAngleEnter() ==-1 && Car->GetOverBridge() && Car->GetLastPort() == 3)
        {
            if(((Car->GetAngle() > 270 || Car->GetAngle() < 90) && (Car->GetLogicalPortVertEnteredAngleLeave() < 270 && Car->GetLogicalPortVertEnteredAngleLeave() > 90)) ||  ((Car->GetAngle() < 270 && Car->GetAngle() > 90) && (Car->GetLogicalPortVertEnteredAngleLeave() > 270 || Car->GetLogicalPortVertEnteredAngleLeave() < 90)))
            {
                printf("6-1\n");
                Car->SetLogicalPortHorzValue(true);
                Car->SetOverBridge(false);
                Car->SetLogicalPortVertEnteredAngleLeave(-1);
               // Car->SetInOnBridge(!Car->GetInOnBridge());
            }
        }



    if(Car->GetOverLogicalPortHorzEnter() && Car->GetLogicalPortHorzEnteredAngleLeave() !=-1 && Car->GetLogicalPortHorzEnteredAngleEnter() !=-1 && Car->GetOverBridge() && Car->GetLastPort() == 2)
    {
        if((Car->GetAngle() > 180 && Car->GetLogicalPortHorzEnteredAngleEnter() > 180) || (Car->GetAngle() < 180 && Car->GetLogicalPortHorzEnteredAngleEnter() < 180))
        {
            printf("7\n");
            Car->SetLogicalPortVertValue(true);
            Car->SetOverBridge(false);
            Car->SetInOnBridge(!Car->GetInOnBridge());
            Car->SetLogicalPortHorzEnteredAngleLeave(-1);

        }
        Car->SetLogicalPortHorzEnteredAngleEnter(-1);

    }


    if(Car->GetOverLogicalPortVertEnter() && Car->GetLogicalPortVertEnteredAngleLeave() !=-1 && Car->GetLogicalPortVertEnteredAngleEnter() !=-1  && Car->GetOverBridge() && Car->GetLastPort() == 1)
    {
        if((Car->GetAngle() <270 && Car->GetAngle() > 90 && Car->GetLogicalPortVertEnteredAngleEnter() < 270 && Car->GetLogicalPortVertEnteredAngleEnter() > 90) || ((Car->GetAngle() >270 || Car->GetAngle() < 90) && (Car->GetLogicalPortVertEnteredAngleEnter() > 270 || Car->GetLogicalPortVertEnteredAngleEnter() < 90)))
        {
            printf("8\n");
            Car->SetLogicalPortHorzValue(true);
            Car->SetOverBridge(false);
            //Car->SetInOnBridge(!Car->GetInOnBridge());
            Car->SetLogicalPortVertEnteredAngleLeave(-1);

        }
        Car->SetLogicalPortVertEnteredAngleEnter(-1);

    }

}
void SwitchLogicalPortsEnter(CCar *Car)
{
    printf("SwitchLogicalPortsEnter\n");
    if(!Car->GetOverLogicalPortHorzEnter() && Car->GetLogicalPortHorzEnteredAngleEnter() !=-1 && Car->GetLogicalPortHorzEnteredAngleLeave() ==-1 && !Car->GetOverBridge() && Car->GetLastPort() == 2)
    {
        //printf("1\n");
        Car->SetLogicalPortVertValue(false);
        Car->SetOverBridge(true);
        Car->SetInOnBridge(!Car->GetInOnBridge());
    }
    else
        if(Car->GetOverLogicalPortHorzEnter() && Car->GetLogicalPortHorzEnteredAngleEnter() !=-1 && Car->GetLogicalPortHorzEnteredAngleLeave() ==-1 && Car->GetOverBridge() && Car->GetLastPort() == 2)
        {
            if((Car->GetAngle() > 180 && Car->GetLogicalPortHorzEnteredAngleEnter() < 180) || (Car->GetAngle() < 180 && Car->GetLogicalPortHorzEnteredAngleEnter() > 180))
            {
                printf("1-1\n");
                Car->SetLogicalPortVertValue(true);
                Car->SetOverBridge(false);
                Car->SetInOnBridge(!Car->GetInOnBridge());
                Car->SetLogicalPortHorzEnteredAngleEnter(-1);
                //Car->SetOverLogicalPortHorzEnter(false);
            }
        }

    if(!Car->GetOverLogicalPortVertEnter() && Car->GetLogicalPortVertEnteredAngleEnter() !=-1 && Car->GetLogicalPortVertEnteredAngleLeave() ==-1 && !Car->GetOverBridge() && Car->GetLastPort() == 1)
    {
        printf("2\n");
        Car->SetLogicalPortHorzValue(false);
        Car->SetOverBridge(true);
       // Car->SetInOnBridge(!Car->GetInOnBridge());
       //Car->SetLogicalPortVertEnteredAngleEnter(-1);
    }
    else
        if(Car->GetOverLogicalPortVertEnter() && Car->GetLogicalPortVertEnteredAngleEnter() !=-1 && Car->GetLogicalPortVertEnteredAngleLeave() ==-1 && Car->GetOverBridge() && Car->GetLastPort() == 1)
        {
            if(((Car->GetAngle() > 270 || Car->GetAngle() < 90) && (Car->GetLogicalPortVertEnteredAngleEnter() < 270 && Car->GetLogicalPortVertEnteredAngleEnter() > 90)) ||  ((Car->GetAngle() < 270 && Car->GetAngle() > 90) && (Car->GetLogicalPortVertEnteredAngleEnter() > 270 || Car->GetLogicalPortVertEnteredAngleEnter() < 90)))
            {
                printf("2-1\n");
                Car->SetLogicalPortHorzValue(true);
                Car->SetOverBridge(false);
                Car->SetLogicalPortVertEnteredAngleEnter(-1);
            }
        }





    if(Car->GetOverLogicalPortHorzLeave() && Car->GetLogicalPortHorzEnteredAngleEnter() !=-1 && Car->GetLogicalPortHorzEnteredAngleLeave() !=-1 && Car->GetOverBridge() && Car->GetLastPort() == 4)
    {
        //printf("kjakaka");
        if((Car->GetAngle() > 180 && Car->GetLogicalPortHorzEnteredAngleLeave() > 180) || (Car->GetAngle() < 180 && Car->GetLogicalPortHorzEnteredAngleLeave() < 180))
        {
            printf("3\n");
            Car->SetLogicalPortVertValue(true);
            Car->SetOverBridge(false);
            Car->SetInOnBridge(!Car->GetInOnBridge());
            Car->SetLogicalPortHorzEnteredAngleEnter(-1);

        }
        Car->SetLogicalPortHorzEnteredAngleLeave(-1);
    }




    if(Car->GetOverLogicalPortVertLeave() && Car->GetLogicalPortVertEnteredAngleEnter() !=-1 && Car->GetLogicalPortVertEnteredAngleLeave() !=-1 && Car->GetOverBridge() && Car->GetLastPort() == 3)
    {
        if((Car->GetAngle() <270 && Car->GetAngle() > 90 && Car->GetLogicalPortVertEnteredAngleLeave() < 270 && Car->GetLogicalPortVertEnteredAngleLeave() > 90) || ((Car->GetAngle() >270 || Car->GetAngle() < 90) && (Car->GetLogicalPortVertEnteredAngleLeave() > 270 || Car->GetLogicalPortVertEnteredAngleLeave() < 90)))
        {
            printf("4\n");
            Car->SetLogicalPortHorzValue(true);
            Car->SetOverBridge(false);

            //Car->SetInOnBridge(!Car->GetInOnBridge());
            Car->SetLogicalPortVertEnteredAngleEnter(-1);
        }
        Car->SetLogicalPortVertEnteredAngleLeave(-1);

    }

}

/*void SwitchLogicalPorts(CCar *Car)
{
    int Teller;
    if(Car->GetLogicalPort1EnteredAngle() != -1 || Car->GetLogicalPort2EnteredAngle() != -1)
        {
            if ((Car->GetLastPort() == 2 ) && !Car->GetOverLogicalPort2())
            {

                if( ((((Car->GetLogicalPort2EnteredAngle() > 180 && Car->GetLogicalPort2EnteredAngle() < 360) && (Car->GetAngle() > 180 && Car->GetAngle() <360 ) && Car->GetLogicalPortHorizontal()) ||

                    ((Car->GetLogicalPort2EnteredAngle() < 180 && Car->GetLogicalPort2EnteredAngle() > -1) && (Car->GetAngle() < 180 && Car->GetAngle() > -1) && Car->GetLogicalPortHorizontal() ) ||

                    ((Car->GetLogicalPort2EnteredAngle() > 90 && Car->GetLogicalPort2EnteredAngle() < 270) && (Car->GetAngle() > 90 && Car->GetAngle() < 270) && !Car->GetLogicalPortHorizontal()) ||

                    ((Car->GetLogicalPort2EnteredAngle() < 90 || Car->GetLogicalPort2EnteredAngle() > 270) && (Car->GetAngle() < 90 || Car->GetAngle() >270) && !Car->GetLogicalPortHorizontal())

                     )))

                {
                        printf("Switch port1 LogicalPort2EnteredAngle: %d CARA:%d\n",Car->GetLogicalPort2EnteredAngle(),Car->GetAngle());

                        Car->SetLogicalPort1Value(!Car->GetLogicalPort1Value());
                        Car->SetOverBridge(!(Car->GetLogicalPort1Value() && Car->GetLogicalPort2Value()));
                        Car->SetInOnBridge(!Car->GetInOnBridge());
                }
                Car->SetLastPort(0);
            }



            if ((Car->GetLastPort() == 1 ) && !Car->GetOverLogicalPort1() )
            {


               if( ((((Car->GetLogicalPort1EnteredAngle() > 180 && Car->GetLogicalPort1EnteredAngle() < 360) && (Car->GetAngle() > 180 && Car->GetAngle() <360 )  && Car->GetLogicalPortHorizontal()) ||

                    ((Car->GetLogicalPort1EnteredAngle() < 180 && Car->GetLogicalPort1EnteredAngle() > -1) && (Car->GetAngle() < 180 && Car->GetAngle() > -1) && Car->GetLogicalPortHorizontal()) ||

                    ((Car->GetLogicalPort1EnteredAngle() > 90 && Car->GetLogicalPort1EnteredAngle() < 270) && (Car->GetAngle() > 90 && Car->GetAngle() < 270) && !Car->GetLogicalPortHorizontal()) ||

                    ((Car->GetLogicalPort1EnteredAngle() < 90 || Car->GetLogicalPort1EnteredAngle() > 270) && (Car->GetAngle() < 90 || Car->GetAngle() >270) && !Car->GetLogicalPortHorizontal())



                    ) ))

                {
                        printf("Switch port2 LogicalPort1EnteredAngle:%d CARA:%d\n",Car->GetLogicalPort1EnteredAngle(),Car->GetAngle());
                        Car->SetLogicalPort2Value(!Car->GetLogicalPort2Value());
                        Car->SetOverBridge(!(Car->GetLogicalPort1Value() && Car->GetLogicalPort2Value()));
                        Car->SetOverBridge(!Car->GetLogicalPort2Value());
                        //Car->SetInOnBridge(!Car->GetInOnBridge());

                }
                Car->SetLastPort(0);


            }

        if(!Car->GetOverLogicalPort2())
            if(Car->GetLogicalPort2EnteredAngle() != -1)
                Car->SetLogicalPort2EnteredAngle(-1);
        if(!Car->GetOverLogicalPort1())
            if(Car->GetLogicalPort1EnteredAngle() != -1)
                Car->SetLogicalPort1EnteredAngle(-1);

        }
}*/

void PlayLevel(CInput *Input, CCar *Car, CCar *ReplayCar,SDL_Surface * Background,SDL_Surface *MiddleLayer,SDL_Surface *TopLayer,SDL_Surface *Mask,CPartsManager *PartsManager,SDL_Surface *Dst)
{
    int PartIndex = -1,Teller;
//    CAiCar *AiCar;
    HandleMusicLoop();
    HandleSound(Car);
    ReplayCar->Move();
    Car->Move();
    Camera->SetPos(Car->GetIntX(),Car->GetIntY());


    //if(Input->KeyboardHeld[SDLK_k])

//    if(!AiCar->GetInOnBridge())
//        AiCar->Hide();
//    else
//        AiCar->UnHide();
   // printf("Carboucing:%d\n",Car->IsBouncing());
    if(TestPixelMask((int)Car->GetNextX(),(int)Car->GetNextY(),Mask)==0)
    {
            PixelBounce = true;
            if(!Car->IsBouncing())
                Car->Bounce();
            else
                Car->StopBounce();
            //OverLogicalPort1 = false;
            //OverLogicalPort2 = false;
            //printf("Testing ports\n");
        //SwitchLogicalPorts(Car);
            TmpX = (int)Car->GetBounceToX();
            TmpY = (int)Car->GetBounceToY();


    }
    else
    {
        if(!Car->IsBouncing())
            PixelBounce=false;
    }


    //printf("ReplayCar X:%f Y:%f\n",ReplayCar->GetX(),ReplayCar->GetX());
   // if(!Car->IsResetting())


    //checks for the position of the car
    PartIndex = PartsManager->CollidePoint((int)(Car->GetIntX()),(int)(Car->GetIntY()));
    if(PartIndex >= 0)
    {

        CPart *Part = PartsManager->Items[PartIndex];



        //printf("Collision\n");
        switch(Part->GetId())
        {

            case DFINISHDOWNID:
                if(Car->CheckPositionCollision(Part->GetLocation()))
                {
                    if(Car->GetAngle() <= 180 )
                    {
                        Car->SetFinishPassed(true);
                        if(Car->GetCheckPoint1Passed() && Car->GetCheckPoint2Passed() && Car->GetFinishPassed() )
                        {


                            Car->SetCheckPoint1Passed(false);
                            Car->SetCheckPoint2Passed(false);
                            if(Car->GetLaps() >= 1)
                            {
                                ReplayCar->UnHide();

                                Uint32 NewTime = SDL_GetTicks();
                                if( NewTime - LapTime < BestLapTime)
                                {
                                    BestLapTime = NewTime - LapTime;
                                  //  printf("Replay size Car: %d\n",Car->GetReplayData().size());
                                    ReplayCar->SetReplayData(Car->GetReplayData());
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                   // printf("Replay size ReplayCar: %d\n",ReplayCar->GetReplayData().size());
                                }
                                else
                                {
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                }
                                LapTime = SDL_GetTicks();

                            }
                            else
                            {
                                Car->EraseReplayData();
                                LapTime = SDL_GetTicks();
                            }
                            Car->SetLaps(Car->GetLaps()+1);
                        }


                    }
                    else
                    {
                        Car->SetCheckPoint1Passed(false);
                        Car->SetCheckPoint2Passed(false);
                        Car->SetFinishPassed(false);
                    }




                    //ReplayCar->ResetReplay();
                }

                break;

            case DFINISHLEFTID:
                if(Car->CheckPositionCollision(Part->GetLocation()))
                {
                    if(Car->GetAngle() >= 90 && Car->GetAngle() <= 270 )
                    {
                        Car->SetFinishPassed(true);
                        if(Car->GetCheckPoint1Passed() && Car->GetCheckPoint2Passed() && Car->GetFinishPassed()  )
                        {


                            Car->SetCheckPoint1Passed(false);
                            Car->SetCheckPoint2Passed(false);
                            if(Car->GetLaps() >= 1)
                            {
                                ReplayCar->UnHide();

                                Uint32 NewTime = SDL_GetTicks();
                                if( NewTime - LapTime < BestLapTime)
                                {
                                    BestLapTime = NewTime - LapTime;
                                  //  printf("Replay size Car: %d\n",Car->GetReplayData().size());
                                    ReplayCar->SetReplayData(Car->GetReplayData());
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                   // printf("Replay size ReplayCar: %d\n",ReplayCar->GetReplayData().size());
                                }
                                else
                                {
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                }
                                LapTime = SDL_GetTicks();

                            }
                            else
                            {
                                Car->EraseReplayData();
                                LapTime = SDL_GetTicks();
                            }
                            Car->SetLaps(Car->GetLaps()+1);
                        }


                    }
                    else
                    {
                        Car->SetCheckPoint1Passed(false);
                        Car->SetCheckPoint2Passed(false);
                        Car->SetFinishPassed(false);
                    }




                    //ReplayCar->ResetReplay();
                }
                break;
            case DFINISHRIGHTID:
               if(Car->CheckPositionCollision(Part->GetLocation()))
                {
                    if(Car->GetAngle() >= 270 || Car->GetAngle() <=90 )
                    {
                        Car->SetFinishPassed(true);
                        if(Car->GetCheckPoint1Passed() && Car->GetCheckPoint2Passed() && Car->GetFinishPassed() )
                        {


                            Car->SetCheckPoint1Passed(false);
                            Car->SetCheckPoint2Passed(false);
                            if(Car->GetLaps() >= 1)
                            {
                                ReplayCar->UnHide();

                                Uint32 NewTime = SDL_GetTicks();
                                if( NewTime - LapTime < BestLapTime)
                                {
                                    BestLapTime = NewTime - LapTime;
                                  //  printf("Replay size Car: %d\n",Car->GetReplayData().size());
                                    ReplayCar->SetReplayData(Car->GetReplayData());
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                   // printf("Replay size ReplayCar: %d\n",ReplayCar->GetReplayData().size());
                                }
                                else
                                {
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                }
                                LapTime = SDL_GetTicks();

                            }
                            else
                            {
                                Car->EraseReplayData();
                                LapTime = SDL_GetTicks();
                            }
                            Car->SetLaps(Car->GetLaps()+1);
                        }


                    }
                    else
                    {
                        Car->SetCheckPoint1Passed(false);
                        Car->SetCheckPoint2Passed(false);
                        Car->SetFinishPassed(false);
                    }




                    //ReplayCar->ResetReplay();
                }
                break;
            case DFINISHUPID :
                if(Car->CheckPositionCollision(Part->GetLocation()))
                {
                    if(Car->GetAngle() >= 180 )
                    {
                        Car->SetFinishPassed(true);
                        if(Car->GetCheckPoint1Passed() && Car->GetCheckPoint2Passed() && Car->GetFinishPassed()  )
                        {


                            Car->SetCheckPoint1Passed(false);
                            Car->SetCheckPoint2Passed(false);
                            if(Car->GetLaps() >= 1)
                            {
                                ReplayCar->UnHide();

                                Uint32 NewTime = SDL_GetTicks();
                                if( NewTime - LapTime < BestLapTime)
                                {
                                    BestLapTime = NewTime - LapTime;
                                  //  printf("Replay size Car: %d\n",Car->GetReplayData().size());
                                    ReplayCar->SetReplayData(Car->GetReplayData());
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                   // printf("Replay size ReplayCar: %d\n",ReplayCar->GetReplayData().size());
                                }
                                else
                                {
                                    ReplayCar->ResetReplay();
                                    Car->EraseReplayData();
                                }
                                LapTime = SDL_GetTicks();

                            }
                            else
                            {
                                Car->EraseReplayData();
                                LapTime = SDL_GetTicks();
                            }
                            Car->SetLaps(Car->GetLaps()+1);
                        }


                    }
                    else
                    {
                        Car->SetCheckPoint1Passed(false);
                        Car->SetCheckPoint2Passed(false);
                        Car->SetFinishPassed(false);
                    }




                    //ReplayCar->ResetReplay();
                }
                break;
            case DCHECKPOINT1HORZID:
            case DCHECKPOINT1VERTID:
                if(Car->CheckPositionCollision(Part->GetLocation()))
                {
                    if(!Car->GetCheckPoint1Passed() && !Car->GetCheckPoint2Passed() && Car->GetFinishPassed())
                    {

                        //CheckPoint1Passed=FinishedPassed;
                      //  CheckPoint2Passed=false;
                      //FinishedPassed =true;
                        Car->SetCheckPoint1Passed(true);

                    }

                }
                break;
            case DCHECKPOINT2HORZID:
            case DCHECKPOINT2VERTID:
                if(Car->CheckPositionCollision(Part->GetLocation()))
                {
                    if(Car->GetCheckPoint1Passed() && !Car->GetCheckPoint2Passed() && Car->GetFinishPassed())
                    {

                        Car->SetCheckPoint2Passed(true);
                        //FinishedPassed = false;
                    }


                }
                break;
            case DLOGICALPORTHORZENTERID:
                printf("DLOGICALPORTHORZENTERID\n");
                if(!Car->GetLogicalPortHorzValue())
                {
                    if(!Car->IsBouncing())
                        Car->Bounce();
                }
                else
                {
                    if(!Car->GetOverLogicalPortHorzEnter())
                    {
                        Car->SetLastPort(2);
                        if(Car->GetLogicalPortHorzEnteredAngleEnter() == -1)
                            Car->SetLogicalPortHorzEnteredAngleEnter(Car->GetAngle());
                        SwitchLogicalPortsEnter(Car);
                        Car->SetOverLogicalPortHorzEnter(true);

                        Car->SetLogicalPortHorizontal(true);

                    }

                }
                break;
            case DLOGICALPORTVERTENTERID:
                if(!Car->GetLogicalPortVertValue())
                {
                    if(!Car->IsBouncing())
                        Car->Bounce();
                }
                else
                {
                    if(!Car->GetOverLogicalPortVertEnter())
                    {
                        Car->SetLastPort(1);
                        if(Car->GetLogicalPortVertEnteredAngleEnter() == -1)
                            Car->SetLogicalPortVertEnteredAngleEnter(Car->GetAngle());
                        SwitchLogicalPortsEnter(Car);
                        Car->SetOverLogicalPortVertEnter(true);

                        Car->SetLogicalPortHorizontal(false);
                        //switchPorts(Car);
                    }
                }
                break;
            case DLOGICALPORTHORZLEAVEID:
                printf("DLOGICALPORTHORZLEAVEID\n");
                if(!Car->GetLogicalPortHorzValue())
                {
                    if(!Car->IsBouncing())
                        Car->Bounce();
                }
                else
                {
                    if(!Car->GetOverLogicalPortHorzLeave())
                    {
                        Car->SetLastPort(4);
                        if(Car->GetLogicalPortHorzEnteredAngleLeave() == -1)
                            Car->SetLogicalPortHorzEnteredAngleLeave(Car->GetAngle());
                        SwitchLogicalPortsLeave(Car);
                        Car->SetOverLogicalPortHorzLeave(true);

                        //Car->SetLogicalPortHorzEnteredAngle(Car->GetAngle());
                        Car->SetLogicalPortHorizontal(true);
                        //switchPorts(Car);
                    }
                }
                break;
            case DLOGICALPORTVERTLEAVEID:
                if(!Car->GetLogicalPortVertValue())
                {
                    if(!Car->IsBouncing())
                        Car->Bounce();
                }
                else
                {
                    if(!Car->GetOverLogicalPortVertLeave())
                    {
                        Car->SetLastPort(3);
                        if(Car->GetLogicalPortVertEnteredAngleLeave() == -1)
                            Car->SetLogicalPortVertEnteredAngleLeave(Car->GetAngle());
                        SwitchLogicalPortsLeave(Car);
                        Car->SetOverLogicalPortVertLeave(true);
                        //Car->SetLogicalPortHorzEnteredAngle(Car->GetAngle());
                        Car->SetLogicalPortHorizontal(false);
                        //switchPorts(Car);
                    }
                }
                break;
         /*   case DLOGICALPORT1VERTID:
                    if(!Car->GetLogicalPort1Value())
                    {
                        if(!Car->IsBouncing())
                            Car->Bounce();


                    }
                    else
                    {
                        if(!Car->GetOverLogicalPort1())
                        {
                            Car->SetOverLogicalPort1(true);
                            Car->SetLogicalPort1EnteredAngle(Car->GetAngle());
                            Car->SetLogicalPortHorizontal(false);
                            Car->SetLastPort(1);
                        }

                         if( Car->GetOverLogicalPort2())
                        {
                            printf("bleh 1\n");
                            Car->SetOverLogicalPort2(false);
                            SwitchLogicalPorts(Car);

                        }
                    }
                    break;

            case DLOGICALPORT1HORZID:
                    if(!Car->GetLogicalPort1Value())
                    {
                        if(!Car->IsBouncing())
                            Car->Bounce();


                    }
                    else
                    {
                        if(!Car->GetOverLogicalPort1())
                        {
                            Car->SetOverLogicalPort1(true);
                            Car->SetLogicalPort1EnteredAngle(Car->GetAngle());
                            Car->SetLogicalPortHorizontal(true);
                            Car->SetLastPort(1);
                        }

                         if( Car->GetOverLogicalPort2())
                        {
                            printf("bleh 2\n");
                            Car->SetOverLogicalPort2(false);
                            SwitchLogicalPorts(Car);

                        }
                    }
                    break;
            case DLOGICALPORT2VERTID:
                   if(!Car->GetLogicalPort2Value())
                    {
                        if(!Car->IsBouncing())
                            Car->Bounce();


                    }
                    else
                    {
                        if(!Car->GetOverLogicalPort2())
                        {
                            Car->SetOverLogicalPort2(true);
                            Car->SetLogicalPort2EnteredAngle(Car->GetAngle());
                            Car->SetLogicalPortHorizontal(false);
                            Car->SetLastPort(2);
                        }
                        if( Car->GetOverLogicalPort1())
                        {
                            printf("bleh 3\n");
                            Car->SetOverLogicalPort1(false);
                            SwitchLogicalPorts(Car);
                        }
                    }
                    break;
            case DLOGICALPORT2HORZID:
                    if(!Car->GetLogicalPort2Value())
                    {
                        if(!Car->IsBouncing())
                            Car->Bounce();


                    }
                    else
                    {
                        if(!Car->GetOverLogicalPort2())
                        {
                            Car->SetOverLogicalPort2(true);
                            Car->SetLogicalPort2EnteredAngle(Car->GetAngle());
                            Car->SetLogicalPortHorizontal(true);
                            Car->SetLastPort(2);
                        }
                        if( Car->GetOverLogicalPort1())
                        {
                            printf("bleh 4\n");
                            Car->SetOverLogicalPort1(false);
                            SwitchLogicalPorts(Car);
                        }
                        break;
                    }*/

           /* case DLOGICALPORT1HORZID:
                TmpHorz1 = (CLogicalPort1Horz*) Part;
                if(!TmpHorz1->GetValue())
                {
                    LogicalPort2EnteredAngle = -1;
                    if(!PixelBounce && !Car->IsBouncing())
                    {
                        Car->Bounce();
                    }
                }
                else
                {
                    if(OverLogicalPort2)
                    {
                        printf("bleh 1\n");
                        OverLogicalPort2 = false;
                        SwitchLogicalPorts(PartsManager,Car);
                    }

                    if(!OverLogicalPort1)
                    {
                       // OverLogicalPort2 = false;
                        LogicalPortHorizontal = true;
                        LastLogicalPortEntered = 1;
                        OverLogicalPort1 = true;
                        LogicalPort1EnteredAngle = Car->GetAngle();
                    }
                }
                break;

            case DLOGICALPORT2HORZID:
                TmpHorz2 = (CLogicalPort2Horz*) Part;
                if(!TmpHorz2->GetValue())
                {
                    LogicalPort1EnteredAngle = -1;
                    if(!PixelBounce && !Car->IsBouncing())
                    {
                        Car->Bounce();
                    }
                }
                else
                {
                    if(OverLogicalPort1)
                    {
                        printf("bleh 2\n");
                        OverLogicalPort1 = false;
                        SwitchLogicalPorts(PartsManager,Car);
                    }

                    if(!OverLogicalPort2)
                    {
                       // OverLogicalPort1 = false;
                        LogicalPortHorizontal = true;
                        LastLogicalPortEntered = 2;
                        OverLogicalPort2 = true;
                        LogicalPort2EnteredAngle = Car->GetAngle();
                    }
                }
                break;

            case DLOGICALPORT1VERTID:
                TmpVert1 = (CLogicalPort1Vert*) Part;
                if(!TmpVert1->GetValue())
                {
                    LogicalPort2EnteredAngle = -1;
                    if(!PixelBounce && !Car->IsBouncing())
                    {
                        Car->Bounce();
                    }
                }
                else
                {
                    if(OverLogicalPort2)
                    {
                        printf("bleh 3\n");
                        OverLogicalPort2 = false;
                        SwitchLogicalPorts(PartsManager,Car);
                    }

                    if(!OverLogicalPort1)
                    {
                        LogicalPortHorizontal = false;
                        LastLogicalPortEntered = 1;
                        OverLogicalPort1 = true;
                        LogicalPort1EnteredAngle = Car->GetAngle();
                    }
                }
                break;

            case DLOGICALPORT2VERTID:
                TmpVert2 = (CLogicalPort2Vert*) Part;
                if(!TmpVert2->GetValue())
                {
                   LogicalPort1EnteredAngle = -1;
                   if(!PixelBounce && !Car->IsBouncing())
                    {
                        Car->Bounce();
                    }
                }
                else
                {
                    if(OverLogicalPort1)
                    {
                        OverLogicalPort1 = false;
                        SwitchLogicalPorts(PartsManager,Car);
                    }

                    if(!OverLogicalPort2)
                    {
                        LogicalPortHorizontal = false;
                        LastLogicalPortEntered = 2;
                        OverLogicalPort2 = true;
                        LogicalPort2EnteredAngle = Car->GetAngle();
                    }
                }
                break;
                    */
            default:

                break;


        }
    }
    else
    {
        //printf("a\n");

        if(Car->GetOverLogicalPortHorzLeave()  && Car->GetLastPort() == 4)
        {
            printf("a1\n");
            SwitchLogicalPortsEnter(Car);
            //Car->SetOverLogicalPortHorzEnter(false);
            //Car->SetOverLogicalPortHorzLeave(false);
        }

        if(Car->GetOverLogicalPortHorzEnter()  && Car->GetLastPort() ==2)
        {
            printf("a2\n");
            SwitchLogicalPortsEnter(Car);
            //Car->SetOverLogicalPortHorzEnter(false);
            //Car->SetOverLogicalPortHorzLeave(false);
        }


        //else
            /*if(Car->GetOverLogicalPortHorzEnter() && Car->GetLastPort() == 2)
            {

                SwitchLogicalPortsEnter(Car);
                //Car->SetOverLogicalPortHorzEnter(false);
               // Car->SetOverLogicalPortHorzLeave(false);
            }*/


        if(Car->GetOverLogicalPortVertLeave() && Car->GetLastPort() == 3)
        {
            printf("a3\n");
            SwitchLogicalPortsEnter(Car);
            //Car->SetOverLogicalPortVertEnter(false);
            //Car->SetOverLogicalPortVertLeave(false);
        }

        if(Car->GetOverLogicalPortVertEnter() && Car->GetLastPort() == 1)
        {
            printf("a4\n");
            SwitchLogicalPortsEnter(Car);
            //Car->SetOverLogicalPortVertEnter(false);
            //Car->SetOverLogicalPortVertLeave(false);
        }



        if(Car->GetOverLogicalPortHorzEnter() && Car->GetLastPort() == 2)
        {
            printf("a5\n");
            SwitchLogicalPortsLeave(Car);
            //Car->SetOverLogicalPortHorzEnter(false);
            //Car->SetOverLogicalPortHorzLeave(false);
        }

        if(Car->GetOverLogicalPortHorzLeave()  && Car->GetLastPort() ==4)
        {
            printf("a6\n");
            SwitchLogicalPortsLeave(Car);
            //Car->SetOverLogicalPortHorzEnter(false);
            //Car->SetOverLogicalPortHorzLeave(false);
        }
       // else




         if( Car->GetOverLogicalPortVertEnter()&& Car->GetLastPort() == 1)
        {
            printf("a7\n");
            SwitchLogicalPortsLeave(Car);
            //Car->SetOverLogicalPortVertEnter(false);
            //Car->SetOverLogicalPortVertLeave(false);
        }

         if( Car->GetOverLogicalPortVertLeave()&& Car->GetLastPort() == 3)
        {
            printf("a8\n");
            SwitchLogicalPortsLeave(Car);
            //Car->SetOverLogicalPortVertEnter(false);
            //Car->SetOverLogicalPortVertLeave(false);
        }

        Car->SetOverLogicalPortHorzEnter(false);
        Car->SetOverLogicalPortHorzLeave(false);
        Car->SetOverLogicalPortVertEnter(false);
        Car->SetOverLogicalPortVertLeave(false);
        //Counter=0;
       // Car->SetOverLogicalPort1(false);
        //Car->SetOverLogicalPort2(false);   //OverLogicalPort2=false;

    }


    int NumSlideDoorsTouched = 0;
    CSlideDoor * SlideDoor;
    //check for Rectangles (mostly important for objects that move etc)
    PartIndex = PartsManager->CollideRect(Car->GetRect(),0);
    while(PartIndex >= 0)
    {

        CPart *Part = PartsManager->Items[PartIndex];



        //printf("Collision\n");
        switch(Part->GetId())
        {
            case DSLIDEDOORUPID:
                if(Car->CheckPixelCollision(Part->GetRealLocation()))
                {
                    SlideDoor = (CSlideDoor*)Part;
                   // Car->SetY(Car->GetY() - SlideDoor->GetDiffY());
                    NumSlideDoorsTouched++;
                    if(Car->GetIntX() >= SlideDoor->GetLocation().x + SlideDoor->GetLocation().w >> 2))
                    {
                        Car->Accelerate();
                        Car->Move();
                    }

                    else
                        if(!Car->IsBouncing())
                            Car->Bounce();

                }
                //Car->SetAngle(Car->GetAngle()+180);
               // if(Car->GetAngle() > 0 && Car->GetAngle() < 180)
                 //   Car->Bounce();
                  //  if(Car->GetSpeed() > Car-GetMaxSpeed()/2)
                    //    Car->ReverseSpeed();

                break;
            case DSLIDEDOORDOWNID:
                if(Car->CheckPixelCollision(Part->GetRealLocation()))
                {
                   SlideDoor = (CSlideDoor*)Part;
                  // Car->SetY(Car->GetY() + SlideDoor->GetDiffY());
                   NumSlideDoorsTouched++;
                   if(Car->GetIntX() >= SlideDoor->GetLocation().x + SlideDoor->GetLocation().w >> 2))
                    {
                        Car->Accelerate();
                        Car->Move();
                    }
                    else
                        if(!Car->IsBouncing())
                            Car->Bounce();

                }
                //if(Car->GetAngle() > 180 && Car->GetAngle() < 360)
                //Car->Bounce();
                //Car->SetAngle(Car->GetAngle()+180);
                break;


            case DSLIDEDOORRIGHTID:
                if(Car->CheckPixelCollision(Part->GetRealLocation()))
                {
                    SlideDoor = (CSlideDoor*)Part;
                   // Car->SetX(Car->GetX() + SlideDoor->GetDiffX());
                    NumSlideDoorsTouched++;
                    if(Car->GetIntY() >= SlideDoor->GetLocation().y + SlideDoor->GetLocation().h >> 2))
                    {
                        Car->Accelerate();
                        Car->Move();
                    }
                    else
                        if(!Car->IsBouncing())
                            Car->Bounce();

                }
                //if(Car->GetAngle() > 90 && Car->GetAngle() < 270)
                //Car->ReverseSpeed();
                //Car->SetAngle(Car->GetAngle()+180);
                break;
            case DSLIDEDOORLEFTID:
                if(Car->CheckPixelCollision(Part->GetRealLocation()))
                {
                    SlideDoor = (CSlideDoor*)Part;
                   // Car->SetX(Car->GetX() - SlideDoor->GetDiffX());
                    NumSlideDoorsTouched++;
                    if(Car->GetIntY() >= SlideDoor->GetLocation().y + SlideDoor->GetLocation().h >> 2))
                    {
                        Car->Accelerate();
                        Car->Move();
                    }
                    else
                        if(!Car->IsBouncing())
                            Car->Bounce();
                }
               // if((Car->GetAngle() > 0 && Car->GetAngle() < 90) ||( Car->GetAngle()  < 360 && Car->GetAngle() >270))
                //Car->Bounce();
                //Car->SetAngle(Car->GetAngle()+180);
                break;
        }
        PartIndex = PartsManager->CollideRect(Car->GetRect(),PartIndex+1);

    }
    if(NumSlideDoorsTouched >= 1)
    {

    }
//    else
//    {
//        if(NumSlideDoorsTouched > 1)
//        {
//            if(!Car->IsBouncing())
//                Car->Bounce();
//            else
//            {
//                Mix_PlayChannel(-1,Sound,0);
//                //Car->ResetCar();
//            }
//        }
//    }

    // printf("Ai car angle: %d\t X: %d\t Y: %d\t Logical port horz enter angle: %d\t Logical port horz leave angle: %d\t Over Logical port horz enter: %d\t OverLogical port Leave: %d\t OverBridge:%d\n",AiCar->GetAngle(),(int)AiCar->GetX(),(int)AiCar->GetY(),AiCar->GetLogicalPortHorzEnteredAngleEnter(),AiCar->GetLogicalPortHorzEnteredAngleLeave(),AiCar->GetOverLogicalPortHorzEnter(),AiCar->GetOverLogicalPortHorzLeave(),AiCar->GetOverBridge());
   // SwitchLogicalPorts(Car);

//    for(Teller = 0;Teller<NUMAICARS;Teller++)
//    {
//        CSlideDoor *SlideDoor;
////        AiCar = (CAiCar4*)AiCars[Teller];
////        AiCar->Move(Buffer,PartsManager,Car,AiCars);
//
//
//        //checks for the position of the car
////        PartIndex = PartsManager->CollidePoint((int)(AiCar->GetX()),(int)(AiCar->GetY()));
//        if(PartIndex >= 0)
//        {
//
//            CPart *Part = PartsManager->Items[PartIndex];
//
//
//
//            //printf("Collision\n");
//            switch(Part->GetId())
//            {
//
//                case DFINISHDOWNID:
//                    if(AiCar->CheckPositionCollision(Part->GetLocation()))
//                    {
//                        if(AiCar->GetAngle() <= 180 )
//                        {
//                            AiCar->SetFinishPassed(true);
//                            if(AiCar->GetCheckPoint1Passed() && AiCar->GetCheckPoint2Passed() && AiCar->GetFinishPassed() )
//                            {
//                                Uint32 NewTime = SDL_GetTicks();
//                                if(AiCar->GetLaps() >= 1)
//                                if( NewTime - LapTime < BestLapTime)
//                                    BestLapTime = NewTime - LapTime;
//                                for(Teller = 0;Teller < PartsManager->ItemCount;Teller++)
//                                {
//                                    switch(PartsManager->Items[Teller]->GetId())
//                                    {
//                                        case DSLIDEDOORDOWNID:
//                                        case DSLIDEDOORRIGHTID:
//                                        case DSLIDEDOORUPID:
//                                        case DSLIDEDOORLEFTID:
//                                            SlideDoor = (CSlideDoor*)PartsManager->Items[Teller];
//                                            SlideDoor->SetAiPassedIt(false);
//                                            break;
//                                        default:
//                                            break;
//                                    }
//                                }
//                                LapTime = SDL_GetTicks();
//                                AiCar->SetCheckPoint1Passed(false);
//                                AiCar->SetCheckPoint2Passed(false);
//
//                                AiCar->SetLaps(AiCar->GetLaps()+1);
//                            }
//
//
//                        }
//                        else
//                        {
//                            AiCar->SetCheckPoint1Passed(false);
//                            AiCar->SetCheckPoint2Passed(false);
//                            AiCar->SetFinishPassed(false);
//                        }
//
//
//
//
//                        //ReplayCar->ResetReplay();
//                    }
//
//                    break;
//
//                case DFINISHLEFTID:
//                    if(AiCar->CheckPositionCollision(Part->GetLocation()))
//                    {
//                        if(AiCar->GetAngle() >= 90 && AiCar->GetAngle() <= 270 )
//                        {
//                            AiCar->SetFinishPassed(true);
//                            if(AiCar->GetCheckPoint1Passed() && AiCar->GetCheckPoint2Passed() && AiCar->GetFinishPassed()  )
//                            {
//                                Uint32 NewTime = SDL_GetTicks();
//                                 if(AiCar->GetLaps() >= 1)
//                                if( NewTime - LapTime < BestLapTime)
//                                    BestLapTime = NewTime - LapTime;
//                                for(Teller = 0;Teller < PartsManager->ItemCount;Teller++)
//                                {
//                                    switch(PartsManager->Items[Teller]->GetId())
//                                    {
//                                        case DSLIDEDOORDOWNID:
//                                        case DSLIDEDOORRIGHTID:
//                                        case DSLIDEDOORUPID:
//                                        case DSLIDEDOORLEFTID:
//                                            SlideDoor = (CSlideDoor*)PartsManager->Items[Teller];
//                                            SlideDoor->SetAiPassedIt(false);
//                                            break;
//                                        default:
//                                            break;
//                                    }
//                                }
//                                LapTime = SDL_GetTicks();
//                                AiCar->SetCheckPoint1Passed(false);
//                                AiCar->SetCheckPoint2Passed(false);
//                                AiCar->SetLaps(AiCar->GetLaps()+1);
//                            }
//
//
//                        }
//                        else
//                        {
//                            AiCar->SetCheckPoint1Passed(false);
//                            AiCar->SetCheckPoint2Passed(false);
//                            AiCar->SetFinishPassed(false);
//                        }
//
//
//
//
//                        //ReplayCar->ResetReplay();
//                    }
//                    break;
//                case DFINISHRIGHTID:
//                   if(AiCar->CheckPositionCollision(Part->GetLocation()))
//                    {
//                        if(AiCar->GetAngle() >= 270 || AiCar->GetAngle() <=90 )
//                        {
//                            AiCar->SetFinishPassed(true);
//                            if(AiCar->GetCheckPoint1Passed() && AiCar->GetCheckPoint2Passed() && AiCar->GetFinishPassed() )
//                            {
//                                Uint32 NewTime = SDL_GetTicks();
//                                 if(AiCar->GetLaps() >= 1)
//                                if( NewTime - LapTime < BestLapTime)
//                                    BestLapTime = NewTime - LapTime;
//                                for(Teller = 0;Teller < PartsManager->ItemCount;Teller++)
//                                {
//                                    switch(PartsManager->Items[Teller]->GetId())
//                                    {
//                                        case DSLIDEDOORDOWNID:
//                                        case DSLIDEDOORRIGHTID:
//                                        case DSLIDEDOORUPID:
//                                        case DSLIDEDOORLEFTID:
//                                            SlideDoor = (CSlideDoor*)PartsManager->Items[Teller];
//                                            SlideDoor->SetAiPassedIt(false);
//                                            break;
//                                        default:
//                                            break;
//                                    }
//                                }
//                                LapTime = SDL_GetTicks();
//                                AiCar->SetCheckPoint1Passed(false);
//                                AiCar->SetCheckPoint2Passed(false);
//                                AiCar->SetLaps(AiCar->GetLaps()+1);
//                            }
//
//
//                        }
//                        else
//                        {
//                            AiCar->SetCheckPoint1Passed(false);
//                            AiCar->SetCheckPoint2Passed(false);
//                            AiCar->SetFinishPassed(false);
//                        }
//
//
//
//
//                        //ReplayCar->ResetReplay();
//                    }
//                    break;
//                case DFINISHUPID :
//                    if(AiCar->CheckPositionCollision(Part->GetLocation()))
//                    {
//                        if(AiCar->GetAngle() >= 180 )
//                        {
//                            AiCar->SetFinishPassed(true);
//                            if(AiCar->GetCheckPoint1Passed() && AiCar->GetCheckPoint2Passed() && AiCar->GetFinishPassed()  )
//                            {
//                                Uint32 NewTime = SDL_GetTicks();
//                                 if(AiCar->GetLaps() >= 1)
//                                if( NewTime - LapTime < BestLapTime)
//                                    BestLapTime = NewTime - LapTime;
//                                for(Teller = 0;Teller < PartsManager->ItemCount;Teller++)
//                                {
//                                    switch(PartsManager->Items[Teller]->GetId())
//                                    {
//                                        case DSLIDEDOORDOWNID:
//                                        case DSLIDEDOORRIGHTID:
//                                        case DSLIDEDOORUPID:
//                                        case DSLIDEDOORLEFTID:
//                                            SlideDoor = (CSlideDoor*)PartsManager->Items[Teller];
//                                            SlideDoor->SetAiPassedIt(false);
//                                            break;
//                                        default:
//                                            break;
//                                    }
//                                }
//                                LapTime = SDL_GetTicks();
//                                AiCar->SetCheckPoint1Passed(false);
//                                AiCar->SetCheckPoint2Passed(false);
//                                AiCar->SetLaps(AiCar->GetLaps()+1);
//                            }
//
//
//                        }
//                        else
//                        {
//                            AiCar->SetCheckPoint1Passed(false);
//                            AiCar->SetCheckPoint2Passed(false);
//                            AiCar->SetFinishPassed(false);
//                        }
//
//
//
//
//                        //ReplayCar->ResetReplay();
//                    }
//                    break;
//                case DCHECKPOINT1HORZID:
//                case DCHECKPOINT1VERTID:
//                    if(AiCar->CheckPositionCollision(Part->GetLocation()))
//                    {
//                        if(!AiCar->GetCheckPoint1Passed() && !AiCar->GetCheckPoint2Passed() && AiCar->GetFinishPassed())
//                        {
//
//                            //CheckPoint1Passed=FinishedPassed;
//                          //  CheckPoint2Passed=false;
//                          //FinishedPassed =true;
//                            AiCar->SetCheckPoint1Passed(true);
//
//                        }
//
//                    }
//                    break;
//                case DCHECKPOINT2HORZID:
//                case DCHECKPOINT2VERTID:
//                    //if(Car->CheckPositionCollision(PartsManager->Items[PartIndex]->GetLocation()))
//                    {
//                        if(AiCar->GetCheckPoint1Passed() && !AiCar->GetCheckPoint2Passed() && AiCar->GetFinishPassed())
//                        {
//
//                            AiCar->SetCheckPoint2Passed(true);
//                            //FinishedPassed = false;
//                        }
//
//
//                    }
//                    break;
//                case DLOGICALPORTHORZENTERID:
//                    printf("DLOGICALPORTHORZENTERID\n");
//                    if(!AiCar->GetLogicalPortHorzValue())
//                    {
//                        if(!AiCar->IsBouncing())
//                            AiCar->Bounce();
//                    }
//                    else
//                    {
//                        if(!AiCar->GetOverLogicalPortHorzEnter())
//                        {
//                            AiCar->SetLastPort(2);
//                            if(AiCar->GetLogicalPortHorzEnteredAngleEnter() == -1)
//                                AiCar->SetLogicalPortHorzEnteredAngleEnter(AiCar->GetAngle());
//                            SwitchLogicalPortsEnter(AiCar);
//                            AiCar->SetOverLogicalPortHorzEnter(true);
//
//                            AiCar->SetLogicalPortHorizontal(true);
//
//                        }
//
//                    }
//                    break;
//                case DLOGICALPORTVERTENTERID:
//                    printf("DLOGICALPORTVERTENTERID\n");
//                    if(!AiCar->GetLogicalPortVertValue())
//                    {
//                        if(!AiCar->IsBouncing())
//                            AiCar->Bounce();
//                    }
//                    else
//                    {
//                        if(!AiCar->GetOverLogicalPortVertEnter())
//                        {
//                            AiCar->SetLastPort(1);
//                            if(AiCar->GetLogicalPortVertEnteredAngleEnter() == -1)
//                                AiCar->SetLogicalPortVertEnteredAngleEnter(AiCar->GetAngle());
//                            SwitchLogicalPortsEnter(AiCar);
//                            AiCar->SetOverLogicalPortVertEnter(true);
//
//                            AiCar->SetLogicalPortHorizontal(false);
//                            //switchPorts(Car);
//                        }
//                    }
//                    break;
//                case DLOGICALPORTHORZLEAVEID:
//                    printf("DLOGICALPORTHORZLEAVEID\n");
//                    if(!AiCar->GetLogicalPortHorzValue())
//                    {
//                        if(!AiCar->IsBouncing())
//                            AiCar->Bounce();
//                    }
//                    else
//                    {
//                        if(!AiCar->GetOverLogicalPortHorzLeave())
//                        {
//                            AiCar->SetLastPort(4);
//                            if(AiCar->GetLogicalPortHorzEnteredAngleLeave() == -1)
//                                AiCar->SetLogicalPortHorzEnteredAngleLeave(AiCar->GetAngle());
//                            SwitchLogicalPortsLeave(AiCar);
//                            AiCar->SetOverLogicalPortHorzLeave(true);
//
//                            //Car->SetLogicalPortHorzEnteredAngle(Car->GetAngle());
//                            AiCar->SetLogicalPortHorizontal(true);
//                            //switchPorts(Car);
//                        }
//                    }
//                    break;
//                case DLOGICALPORTVERTLEAVEID:
//                    printf("DLOGICALPORTVERTLEAVEID\n");
//                    if(!AiCar->GetLogicalPortVertValue())
//                    {
//                        if(!AiCar->IsBouncing())
//                            AiCar->Bounce();
//                    }
//                    else
//                    {
//                        if(!AiCar->GetOverLogicalPortVertLeave())
//                        {
//                            AiCar->SetLastPort(3);
//                            if(AiCar->GetLogicalPortVertEnteredAngleLeave() == -1)
//                                AiCar->SetLogicalPortVertEnteredAngleLeave(AiCar->GetAngle());
//                            SwitchLogicalPortsLeave(AiCar);
//                            AiCar->SetOverLogicalPortVertLeave(true);
//                            //Car->SetLogicalPortHorzEnteredAngle(Car->GetAngle());
//                            AiCar->SetLogicalPortHorizontal(false);
//                            //switchPorts(Car);
//                        }
//                    }
//                    break;

         /*       case DLOGICALPORT1VERTID:
                        if(!AiCar->GetLogicalPort1Value())
                        {
                            if(!AiCar->IsBouncing())
                                AiCar->Bounce();


                        }
                        else
                        {

                            if(!AiCar->GetOverLogicalPort1())
                            {
                              //  AiCar->SetInOnBridge(true);
                                AiCar->SetOverLogicalPort1(true);
                                AiCar->SetLogicalPort1EnteredAngle(AiCar->GetAngle());
                                AiCar->SetLogicalPortHorizontal(false);
                                AiCar->SetLastPort(1);
                            }

                             if( AiCar->GetOverLogicalPort2())
                            {
                                printf("bleh 1\n");
                                AiCar->SetOverLogicalPort2(false);
                                SwitchLogicalPorts(AiCar);

                            }
                        }
                        break;

                case DLOGICALPORT1HORZID:
                        if(!AiCar->GetLogicalPort1Value())
                        {
                            if(!AiCar->IsBouncing())
                                AiCar->Bounce();


                        }
                        else
                        {

                            if(!AiCar->GetOverLogicalPort1())
                            {
                               // AiCar->SetInOnBridge(true);
                                AiCar->SetOverLogicalPort1(true);
                                AiCar->SetLogicalPort1EnteredAngle(AiCar->GetAngle());
                                AiCar->SetLogicalPortHorizontal(true);
                                AiCar->SetLastPort(1);
                            }

                             if( AiCar->GetOverLogicalPort2())
                            {
                                printf("bleh 2\n");
                                AiCar->SetOverLogicalPort2(false);
                                SwitchLogicalPorts(AiCar);

                            }
                        }
                        break;
                case DLOGICALPORT2VERTID:
                       if(!AiCar->GetLogicalPort2Value())
                        {
                            if(!AiCar->IsBouncing())
                                AiCar->Bounce();


                        }
                        else
                        {

                            if(!AiCar->GetOverLogicalPort2())
                            {
                                //AiCar->SetInOnBridge(false);
                                AiCar->SetOverLogicalPort2(true);
                                AiCar->SetLogicalPort2EnteredAngle(AiCar->GetAngle());
                                AiCar->SetLogicalPortHorizontal(false);
                                AiCar->SetLastPort(2);
                            }
                            if( AiCar->GetOverLogicalPort1())
                            {
                                printf("bleh 3\n");
                                AiCar->SetOverLogicalPort1(false);
                                SwitchLogicalPorts(AiCar);
                            }
                        }
                        break;
                case DLOGICALPORT2HORZID:
                        if(!AiCar->GetLogicalPort2Value())
                        {
                            if(!AiCar->IsBouncing())
                                AiCar->Bounce();


                        }
                        else
                        {

                            if(!AiCar->GetOverLogicalPort2())
                            {
                               // AiCar->SetInOnBridge(false);
                                AiCar->SetOverLogicalPort2(true);
                                AiCar->SetLogicalPort2EnteredAngle(AiCar->GetAngle());
                                AiCar->SetLogicalPortHorizontal(true);
                                AiCar->SetLastPort(2);
                            }
                            if( AiCar->GetOverLogicalPort1())
                            {
                                printf("bleh 4\n");
                                AiCar->SetOverLogicalPort1(false);
                                SwitchLogicalPorts(AiCar);
                            }
                            break;
                        }*/

               /* case DLOGICALPORT1HORZID:
                    TmpHorz1 = (CLogicalPort1Horz*) Part;
                    if(!TmpHorz1->GetValue())
                    {
                        LogicalPort2EnteredAngle = -1;
                        if(!PixelBounce && !Car->IsBouncing())
                        {
                            Car->Bounce();
                        }
                    }
                    else
                    {
                        if(OverLogicalPort2)
                        {
                            printf("bleh 1\n");
                            OverLogicalPort2 = false;
                            SwitchLogicalPorts(PartsManager,Car);
                        }

                        if(!OverLogicalPort1)
                        {
                           // OverLogicalPort2 = false;
                            LogicalPortHorizontal = true;
                            LastLogicalPortEntered = 1;
                            OverLogicalPort1 = true;
                            LogicalPort1EnteredAngle = Car->GetAngle();
                        }
                    }
                    break;

                case DLOGICALPORT2HORZID:
                    TmpHorz2 = (CLogicalPort2Horz*) Part;
                    if(!TmpHorz2->GetValue())
                    {
                        LogicalPort1EnteredAngle = -1;
                        if(!PixelBounce && !Car->IsBouncing())
                        {
                            Car->Bounce();
                        }
                    }
                    else
                    {
                        if(OverLogicalPort1)
                        {
                            printf("bleh 2\n");
                            OverLogicalPort1 = false;
                            SwitchLogicalPorts(PartsManager,Car);
                        }

                        if(!OverLogicalPort2)
                        {
                           // OverLogicalPort1 = false;
                            LogicalPortHorizontal = true;
                            LastLogicalPortEntered = 2;
                            OverLogicalPort2 = true;
                            LogicalPort2EnteredAngle = Car->GetAngle();
                        }
                    }
                    break;

                case DLOGICALPORT1VERTID:
                    TmpVert1 = (CLogicalPort1Vert*) Part;
                    if(!TmpVert1->GetValue())
                    {
                        LogicalPort2EnteredAngle = -1;
                        if(!PixelBounce && !Car->IsBouncing())
                        {
                            Car->Bounce();
                        }
                    }
                    else
                    {
                        if(OverLogicalPort2)
                        {
                            printf("bleh 3\n");
                            OverLogicalPort2 = false;
                            SwitchLogicalPorts(PartsManager,Car);
                        }

                        if(!OverLogicalPort1)
                        {
                            LogicalPortHorizontal = false;
                            LastLogicalPortEntered = 1;
                            OverLogicalPort1 = true;
                            LogicalPort1EnteredAngle = Car->GetAngle();
                        }
                    }
                    break;

                case DLOGICALPORT2VERTID:
                    TmpVert2 = (CLogicalPort2Vert*) Part;
                    if(!TmpVert2->GetValue())
                    {
                       LogicalPort1EnteredAngle = -1;
                       if(!PixelBounce && !Car->IsBouncing())
                        {
                            Car->Bounce();
                        }
                    }
                    else
                    {
                        if(OverLogicalPort1)
                        {
                            OverLogicalPort1 = false;
                            SwitchLogicalPorts(PartsManager,Car);
                        }

                        if(!OverLogicalPort2)
                        {
                            LogicalPortHorizontal = false;
                            LastLogicalPortEntered = 2;
                            OverLogicalPort2 = true;
                            LogicalPort2EnteredAngle = Car->GetAngle();
                        }
                    }
                    break;
                        */
//                default:

//                    break;

//
//            }
//        }
//        else
//        {
//           // printf("b\n");
//            if(AiCar->GetOverLogicalPortHorzLeave()  && AiCar->GetLastPort() == 4)
//            {
//                printf("b1\n");
//                SwitchLogicalPortsEnter(AiCar);
//                //Car->SetOverLogicalPortHorzEnter(false);
//                //Car->SetOverLogicalPortHorzLeave(false);
//            }
//
//            if(AiCar->GetOverLogicalPortHorzEnter()  && AiCar->GetLastPort() ==2)
//            {
//                printf("b2\n");
//                SwitchLogicalPortsEnter(AiCar);
//                //Car->SetOverLogicalPortHorzEnter(false);
//                //Car->SetOverLogicalPortHorzLeave(false);
//            }
//
//
//            //else
//                /*if(Car->GetOverLogicalPortHorzEnter() && Car->GetLastPort() == 2)
//                {
//
//                    SwitchLogicalPortsEnter(Car);
//                    //Car->SetOverLogicalPortHorzEnter(false);
//                   // Car->SetOverLogicalPortHorzLeave(false);
//                }*/
//
//
//            if(AiCar->GetOverLogicalPortVertLeave() && AiCar->GetLastPort() == 3)
//            {
//                printf("b3\n");
//                SwitchLogicalPortsEnter(AiCar);
//                //Car->SetOverLogicalPortVertEnter(false);
//                //Car->SetOverLogicalPortVertLeave(false);
//            }
//
//            if(AiCar->GetOverLogicalPortVertEnter() && AiCar->GetLastPort() == 1)
//            {
//                printf("b4\n");
//                SwitchLogicalPortsEnter(AiCar);
//                //Car->SetOverLogicalPortVertEnter(false);
//                //Car->SetOverLogicalPortVertLeave(false);
//            }
//
//
//
//            if(AiCar->GetOverLogicalPortHorzEnter() && AiCar->GetLastPort() == 2)
//            {
//                printf("b5\n");
//                SwitchLogicalPortsLeave(AiCar);
//                //Car->SetOverLogicalPortHorzEnter(false);
//                //Car->SetOverLogicalPortHorzLeave(false);
//            }
//
//            if(AiCar->GetOverLogicalPortHorzLeave()  && AiCar->GetLastPort() ==4)
//            {
//                printf("b6\n");
//
//                SwitchLogicalPortsLeave(AiCar);
//                //Car->SetOverLogicalPortHorzEnter(false);
//                //Car->SetOverLogicalPortHorzLeave(false);
//            }
//           // else
//
//
//
//
//             if( AiCar->GetOverLogicalPortVertEnter()&& AiCar->GetLastPort() == 1)
//            {
//                printf("b7\n");
//                SwitchLogicalPortsLeave(AiCar);
//                //Car->SetOverLogicalPortVertEnter(false);
//                //Car->SetOverLogicalPortVertLeave(false);
//            }
//
//             if( AiCar->GetOverLogicalPortVertLeave()&& AiCar->GetLastPort() == 3)
//            {
//                printf("b8\n");
//                SwitchLogicalPortsLeave(AiCar);
//                //Car->SetOverLogicalPortVertEnter(false);
//                //Car->SetOverLogicalPortVertLeave(false);
//            }
//
//            AiCar->SetOverLogicalPortHorzEnter(false);
//            AiCar->SetOverLogicalPortHorzLeave(false);
//            AiCar->SetOverLogicalPortVertEnter(false);
//            AiCar->SetOverLogicalPortVertLeave(false);
//            //Counter=0;
//           // AiCar->SetOverLogicalPort1(false);
//           // AiCar->SetOverLogicalPort2(false);   //OverLogicalPort2=false;
//
//        }
//    }

//        if (Car->CheckMaskCollision(AiCars[0]->GetRect()) && AiCars[0]->CheckMaskCollision(Car->GetRect()))
//        {
//            int Speed;
//            Car->SetCollisionAngle(AiCars[0]);
//            //Speed = Car->GetSpeed();
//            //Car->SetSpeed( AiCars[0]->GetSpeed() - Car->GetSpeed());
//            //AiCars[0]->SetSpeed(Speed -AiCars[0]->GetSpeed() );
//           // Car->Bounce();
//           // AiCars[0]->Bounce();
//        }

   // printf("Car Angle:%3d \tLogical Port1 angle : %3d \tLogcal port 2 Angle :%3d \tOVer1:%3d \tOver2:%3d \tHorizontal:%3d \tLast Logical : %3d \tCARX:%3d \tCARY:%3d \tBoucing: %d \tport Horz 1:%3d \tPort Horz2:%3d \tPort Vert1:%3d \tPort Vert2:%3d\t\n ",Car->GetAngle(),LogicalPort1EnteredAngle,LogicalPort2EnteredAngle,OverLogicalPort1,OverLogicalPort2,LogicalPortHorizontal,LastLogicalPortEntered,(int)Car->GetX(),(int)Car->GetY(),Car->IsBouncing(),PortHorz1->GetValue(),PortHorz2->GetValue(),PortVert1->GetValue(),PortVert2->GetValue());
//    SwitchLogicalPorts(AiCar);








}

void LevelSelect()
{
    GLevel+=1;
    if(GLevel > 8)
        GLevel = 1;
    BestLapTime =1000000;
    LapTime = 0;
    GCar->SetLaps(0);
    GameState = GSTest;
}

void Test()
{

    //CCar *Car = new CCar("./graphics/greencar.png",280,170,270,2.5,0.1,0.008);
    //CCar *ReplayCar = new CReplayCar("./graphics/greencarreplay.png",280,145,270);

    CInput *Input = new CInput(25);
    Uint32 StartTimer=0,StartTimerAi = 0;
    int LightNr = 0,AiStartNr=0;
    LoadLevel(GLevel);
    SDL_Rect RectLights,RectBigLight;
    SDL_Surface *Lights,*GreenLight,*RedLight,*OrangeLight,*Tmp;

    Tmp = IMG_Load("./graphics/startlight.png");
    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Tmp->format,255,0,255));
    Lights = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    Tmp = IMG_Load("./graphics/green.png");
    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Tmp->format,255,0,255));
    GreenLight = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    Tmp = IMG_Load("./graphics/red.png");
    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Tmp->format,255,0,255));
    RedLight = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    Tmp = IMG_Load("./graphics/orange.png");
    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY | SDL_RLEACCEL,SDL_MapRGB(Tmp->format,255,0,255));
    OrangeLight = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    //level 1
    bool Starting = true;
    RectLights.x = 320;
    RectLights.y = 200;
    RectLights.w = Lights->w;
    RectLights.h = Lights->h;
    StartTimer = 0;
//    int counter=0;
    LapTime = SDL_GetTicks();

    while(GameState == GSTest)
    //for(int Teller2 = 0; Teller2< NrOfLoops;Teller2++)
    {

        if(!Starting)
        {
            GPartsManager->Move();
        }
        HandleInput(Input);
        DrawLevel(GCar,GReplayCar,LevelBackground,LevelMiddle,LevelTop,Buffer,GPartsManager);
        if(Starting)
        {
            SDL_BlitSurface(Lights,NULL,Buffer,&RectLights);
            if(LightNr == 0)
            {
                RectLights.x = RectLights.x - 1;
                if(RectLights.x == 295)
                {
                    LightNr++;
                    StartTimer = SDL_GetTicks();
                }
            }
            else
            {
                if(SDL_GetTicks()  > StartTimer+750)
                {
                    LightNr++;
                    StartTimer = SDL_GetTicks();
                }
                RectBigLight.x = 295+7 - (RedLight->w / 2);
                RectBigLight.y = 200+6 - (RedLight->h/2) + ((LightNr-2)*8);
                RectBigLight.w = RedLight->w;
                RectBigLight.h = RedLight->h;
                switch (LightNr)
                {
                    case 2:
                        SDL_BlitSurface(RedLight,NULL,Buffer,&RectBigLight);
                        break;
                    case 3:
                        SDL_BlitSurface(RedLight,NULL,Buffer,&RectBigLight);
                        break;
                    case 4:
                        SDL_BlitSurface(OrangeLight,NULL,Buffer,&RectBigLight);
                        break;
                    case 5:
                        SDL_BlitSurface(GreenLight,NULL,Buffer,&RectBigLight);
                        GCar->SetCanMove(true);
                        break;
                    case 6:
                        Starting = false;

                }
            }

        }
        if(!Starting)
        {
            if(RectLights.x < 320)
            {
                RectLights.x += 1;
                SDL_BlitSurface(Lights,NULL,Buffer,&RectLights);
            }
        }
//        if(LightNr == 5 || LightNr == 6)
//        {
//            if(SDL_GetTicks() >  StartTimerAi + 200 )
//            {
//                if(AiStartNr < NUMAICARS)
//                {
//            //        GAiCars[AiStartNr]->SetCanMove(true);
//            //        AiStartNr++;
//                }
//            //    StartTimerAi = SDL_GetTicks();
//            }
//        }

        PlayLevel(Input,GCar,GReplayCar,LevelBackground,LevelMiddle,LevelTop,LevelMask,GPartsManager,Buffer);
        //AiCar->Move(LevelMask,Buffer,PartsManager);
        //AiCar->Draw(Buffer);
        sprintf(FpsText,"FPS:%d",Fps);
        WriteText(Buffer,font,FpsText,strlen(FpsText),0,0,0, FpsTextColor,false);
        ScreenRect.x = StartScreenX;
        ScreenRect.y = StartScreenY;
        ScreenRect.w = Buffer->w;
        ScreenRect.h = Buffer->h;
        SDL_BlitSurface(Buffer,NULL,Screen,&ScreenRect);
//        sprintf(FpsText,"./export/file%d.bmp",counter);
//        SDL_SaveBMP(Buffer,FpsText);
        SDL_Flip(Screen);
//        counter++;

        FrameCounter++;
        if(FPSTicks + 1000 < SDL_GetTicks())
        {
            FPSTicks = SDL_GetTicks();
            Fps = FrameCounter;
            FrameCounter = 0;
        }

        //printf("%d %d\n",StartX,StartY);
       SDL_framerateDelay(&Fpsman);
    }
    SDL_FreeSurface(Lights);
    SDL_FreeSurface(GreenLight);
    SDL_FreeSurface(OrangeLight);
    SDL_FreeSurface(RedLight);
    //GameState = GSQuit;
    delete Input;
}





void LoadSounds()
{
    Music= Mix_LoadMUS("./music/title.mod");
    Sound = Mix_LoadWAV("./sounds/explosion.wav");
    EngineSounds[0] = Mix_LoadWAV("./sounds/engine1.wav");
    EngineSounds[1] = Mix_LoadWAV("./sounds/engine2.wav");
    EngineSounds[2] = Mix_LoadWAV("./sounds/engine3.wav");
    EngineSounds[3] = Mix_LoadWAV("./sounds/engine4.wav");
    EngineSounds[4] = Mix_LoadWAV("./sounds/engine5.wav");
    EngineSounds[5] = Mix_LoadWAV("./sounds/engine6.wav");
    EngineSounds[6] = Mix_LoadWAV("./sounds/engine7.wav");
    EngineSounds[7] = Mix_LoadWAV("./sounds/engine8.wav");
    EngineSounds[8] = Mix_LoadWAV("./sounds/engine9.wav");
    EngineSounds[9] = Mix_LoadWAV("./sounds/engine10.wav");
}

void UnLoadSounds()
{
    Mix_HaltMusic();
    Mix_FreeMusic(Music);
    Mix_FreeChunk(Sound);
    Mix_FreeChunk(EngineSounds[0]);
    Mix_FreeChunk(EngineSounds[1]);
    Mix_FreeChunk(EngineSounds[2]);
    Mix_FreeChunk(EngineSounds[3]);
    Mix_FreeChunk(EngineSounds[4]);
    Mix_FreeChunk(EngineSounds[5]);
    Mix_FreeChunk(EngineSounds[6]);
    Mix_FreeChunk(EngineSounds[7]);
    Mix_FreeChunk(EngineSounds[8]);
    Mix_FreeChunk(EngineSounds[9]);

}

int main(int argc, char *argv[])
{
    // get starting time
   // struct timespec start, stop;
   // clock_gettime(CLOCK_REALTIME,&start);
    SDL_Surface *Tmp;
    SDL_Rect Rect;
#ifndef GP2X
/*	SDL_Surface *Icon;
        if (argc ==2)
        {
            SCREENPOSMULTIPLY = atoi(argv[1]);
            if((SCREENPOSMULTIPLY < 1) || (SCREENPOSMULTIPLY > 10))
                SCREENPOSMULTIPLY = 1;
            WINDOW_WIDTH = 320 *SCREENPOSMULTIPLY,
            WINDOW_HEIGHT = 240*SCREENPOSMULTIPLY,
            TileWidth = 16*SCREENPOSMULTIPLY,
            TileHeight = 16*SCREENPOSMULTIPLY;
        }*/
#endif
	srand((int) time(NULL));
	if (SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == 0)
	{
		//printf("SDL Succesfully initialized\n");
#ifdef GP2X
		if(SDL_GP2X_GetPhysicalScreenSize(&Rect) == 0)
            Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_SWSURFACE  );
        else
        {
            TvOutMode = true;
            Screen = SDL_SetVideoMode( 360, 288,16, SDL_SWSURFACE );
        }
#else
 //       Icon = IMG_Load("./graphics/blockdude.png");
   //     SDL_WM_SetIcon(Icon,NULL);
        Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_SWSURFACE );
     //   SDL_WM_SetCaption( "Blockdude 1.3 - Created By Willems Soft", NULL );*/
#endif
		if(Screen)
		{
		  //  printf("Succesfully Set %dx%dx16\n",WINDOW_WIDTH,WINDOW_HEIGHT);
		    Tmp = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			Buffer = SDL_DisplayFormat(Tmp);
			SDL_FreeSurface(Tmp);
			/*if (TvOutMode)
                Tmp = SDL_CreateRGBSurface(SDL_SWSURFACE,360,288,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
            else
                Tmp = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH, WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
            Buffer1 = SDL_DisplayFormat(Tmp);
            SDL_FreeSurface(Tmp);*/
			if (Buffer)
			{
			    SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
                SDL_ShowCursor(SDL_DISABLE);
#ifdef GP2X
                    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,128) < 0)
#else
                    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,2048) < 0)
#endif
                    {
                        GlobalSoundEnabled = false;
                        printf("Failed to initialise sound!\n");
                        printf("%s\n",Mix_GetError());
                    }
                    else
                    {
            //            printf("Audio Succesfully initialised!\n");

                        if (TTF_Init() == 0)
                        {
              //              printf("Succesfully initialized TTF\n");
                            font = TTF_OpenFont("./font.ttf",15);
                            if (font)
                            {
                //                printf("Succesfully Loaded fonts\n");
                                SDL_initFramerate(&Fpsman);
                                SDL_setFramerate(&Fpsman,FPS);
                                TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
                                LoadSounds();
                                LoadSettings();
                                while (GameState != GSQuit)
                                {
                                    switch(GameState)
                                    {
                                        case GSTest :
                                            Test();
                                            break;
                                        case GSLevelSelect:
                                            LevelSelect();
                                            break;
                                        default:
                                            break;
                                    }
                                }
                                SaveSettings();
                                UnLoadSounds();
                                TTF_CloseFont(font);
                                font=NULL;
                                UnLoadLevel();
                                delete GPartsManager;
                            }
                            else
                            {
                                printf("Failed to Load fonts\n");
                            }
                            TTF_Quit();
                        }
                        else
                        {
                            printf("Failed to initialize TTF\n");
                        }
                        Mix_CloseAudio();
                    }

                SDL_FreeSurface(Buffer);
                Buffer=NULL;
               // SDL_FreeSurface(Buffer1);
                //Buffer1=NULL;
			}
			else
                printf("Failed To Create Buffer\n");
#ifndef GP2X
         //   SDL_FreeSurface(Icon);
#endif
            SDL_FreeSurface(Screen);
            Screen=NULL;
		}
		else
		{
			printf("Failed to Set Videomode %dx%dx16\n",WINDOW_WIDTH, WINDOW_HEIGHT);
		}

		SDL_Quit();
	}
	else
	{
		printf("Couldn't initialise SDL!\n");
	}
    //clock_gettime(CLOCK_REALTIME,&stop);
   // double run_time = (stop.tv_sec - start.tv_sec) + (double)(stop.tv_nsec - start.tv_nsec) / 1000000000.0;
    //fprintf(stdout,"%f",run_time);
	return 0;

}
